/*
ae.cpp

Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com
Subcontract/D.O. DDL 04-D-4032-010
DDL Omni Engineering
Prime Contract N00178-04-D-4032, D.O. FD02
NSWCCD Code 653

JHSV Analysis Engine - NSWCCD-65-TR-2006/15

This program performs statistical analysis on SCL binary data files. This program was originally written
to support the JHSV hull monitoring system, but its analysis capabitlities are useful far beyond the scope of
the project.

If you are attempting to modify this code, it is best to start with the main() function in order to uderstand
what the program does from a high-level standpoint first.
*/

/* TODO

*/


/*
Dump of file ae.exe

File Type: EXECUTABLE IMAGE

  Image has the following dependencies:

    MSVCP80.dll
    MSVCR80.dll
    KERNEL32.dll
*/

#include"ae.h"
string version="200701101306";//version of this code YYYYMMDDHHMM


//accumulates damage of varying stresses/strains
//http://en.wikipedia.org/wiki/Metal_fatigue
void applyMinersRule(int channel){
	//define the SN curve for the desired material
	//like brittle aluminum with uts of 320MPa
	//320,1e0
	//300,1e1
	//260,1e2
	//200,1e3
	//150,1e4
	//120,1e5
	//95,1e6
	//80,1e7
	//50,1e8

	//apply Miner's Rule and correlate the damage at each stress level
	double C=0, amplitude, ub, ubx, lb, lbx, m, b, N;
	for(unsigned int i=0; i < rawcycles[channel].size(); i++){
		amplitude = rawcycles[channel][i];
		if(amplitude >= sncurves[channel][0]) C = 1;
		else if(amplitude >= sncurves[channel][8]){
			//find number of cycles asociated with amplitude
			for(unsigned int j=0; j < 8; j++){
				if(amplitude <= sncurves[channel][j] && amplitude > sncurves[channel][j+1]){
					ub = sncurves[channel][j];
					ubx = pow(10.0,(int)j);
					lb = sncurves[channel][j+1];
					lbx = pow(10.0,(int)j+1);
					break;
				}//end if
			}//end for
			m = (ub-lb)/(ubx-lbx);
			b = ub-m*ubx;
			N = (amplitude-b)/m;
			C = C + 1/N;
		}//end if
	}//end for

	damage[channel] = C;

// just for testing	damage[channel] = 0.123 + channel;
} //applyMinersRule





//this function stores the accumulated damage in
//a binary file of doubles for maximum precision
void accumulatedamagereportbinary(void){

	string filename;
	char temp[256];
	filename = "damagesum-";
	sprintf(temp,"%i",numchan);
	filename.append(temp);
	filename.append("-");
	sprintf(temp,"%i",(int)scanrate);
	filename.append(temp);
	filename.append(".double");

	long filesize;
	ifstream file (filename.data(), ios::in|ios::binary);
	file.seekg (0, ios::end);
	filesize = file.tellg();
	file.close();
	//cout << "filesize: " << filesize << endl;

	double datapt;

	if(filesize > 0){
		//read existing file if it exists
		ifstream inputfile;
		inputfile.open(filename.data(), ios::in|ios::binary);

		inputfile.read((char*)&datapt,8);
		if(datapt != numchan){
			cout << "Error! numchan: " << numchan << " does not match stored numchan: " << datapt << "!\n";
			cerr << "Error! numchan: " << numchan << " does not match stored numchan: " << datapt << "!\n";
			inputfile.close();
			return;
		}//end if

		for(int i=0; i < numchan; i++){
			inputfile.read((char*)&datapt,8);
			damage[i] += datapt;
		}//end for

		inputfile.close();

	}//end if

	ofstream outputfile;
	outputfile.open(filename.data(), ios::trunc|ios::binary);

	double tempnumchan = (double)numchan;

	outputfile.write((char*)&tempnumchan,8);

	for(int i=0; i < numchan; i++){

		outputfile.write((char*)&damage[i],8);

	}//end for

	outputfile.close();

}//end accumulatedamagereportbinary


//this function stores a copy of the accumulated damage
//in an ascii file that can be easily read by other programs
void accumulatedamagereportascii(void){

	string filename;
	char temp[256];
	filename = "damagesum-";
	sprintf(temp,"%i",numchan);
	filename.append(temp);
	filename.append("-");
	sprintf(temp,"%i",(int)scanrate);
	filename.append(temp);
	filename.append(".csv");

	ofstream outputfile;
	outputfile.open(filename.data(), ios::trunc);

	outputfile << numchan << endl;

	for(int i=0; i < numchan; i++){

		outputfile << damage[i] << endl;

	}//end for

	outputfile.close();

}//end accumulatedamagereportascii


void computefullcyclehistogram(int channel){
	fullcyclehistograms[channel].resize(bins);
	float max = maxescycles[channel];
	int histindex = 0;
	float value = 0;
	for(unsigned int c=0; c < rawcycles[channel].size(); c++){
		value = rawcycles[channel][c];
		histindex = (int)ceil(value/max*(float)bins)-1;
		fullcyclehistograms[channel][histindex] += 1;
	}//end for
}//end computefullcyclehistogram


//This function outputs the full cycle histogram for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output psd file name is the same as the input file name
//with -fch added between the name and extension.
//Example: input: run_77.scl outupt: run_77-fch.scl
//The resulting scl file can be loaded directly into IGOR.
//The SCL cal factor for each channel will be the max value in the histogram.
void outputfullcyclehistogram(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -fch added
	string fname = filename;
	fname[fname.size()-4] = '-';
	fname[fname.size()-3] = 'f';
	fname[fname.size()-2] = 'c';
	fname[fname.size()-1] = 'h';
	fname.append(".scl");

	//open and overwrite any existing binary output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc|ios::binary);

	//scl header number of channels
	int tempnumchan = swapbytes(numchan);
	outputfile.write((char*)&tempnumchan,4);

	//scl header scan rate
	float tempscanrate = swapbytes((float)1);
	outputfile.write((char*)&tempscanrate,4);

	//scl header cal factors are the max value in the histogram for each channel
	double tempcal;
	for(i=0; i < (unsigned int)numchan; i++){
		tempcal = (double)maxescycles[i];
		tempcal = swapbytes(tempcal);
		outputfile.write((char*)&tempcal,8);
	}//end for

	//scl interleaved data (actual histograms)
	for(i=0; i < (unsigned)bins; i++){
		for(j=0; j < (unsigned int)numchan; j++){
			if(fullcyclehistograms[j].size() == bins)datapt = swapbytes(fullcyclehistograms[j][i]);
			else datapt=swapbytes((float)0);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputfullcyclehistogram


//this function appends the mmm stats and rainflow damage stats and weibull stats
//to a log file
void outputstatsreport(string filename){
	time_t analysistime;
	time(&analysistime);
	string datechars = ctime(&analysistime);
	datechars[datechars.size()-1] = ',';

	long filesize;
	ifstream file ("statslog.csv", ios::in|ios::binary);
	file.seekg (0, ios::end);
	filesize = file.tellg();
	file.close();
	//cout << "filesize: " << filesize << endl;

	ofstream outputfile;
	outputfile.open("statslog.csv", ios::app);

	if(filesize <=0){//output column headers/labels if this is the first time
		outputfile << "filename,";
		outputfile << "numchan,";
		outputfile << "virtualnumchan,";
		outputfile << "scanrate,";
		outputfile << "analysisdate,";
		outputfile << "analysistime,";
		outputfile << "numscans,";
		outputfile << "seconds,";
		outputfile << "version,";
		outputfile << "fftsize,";
		outputfile << "wavechannel,";
		outputfile << "cutofffrequency,";

		outputfile << "channel,";
		outputfile << "cal,";
		outputfile << "scalefactor,";
		outputfile << "offset,";
		outputfile << "channelname,";

		outputfile << "mean,";
		outputfile << "max,";
		outputfile << "min,";
		outputfile << "variance,";
		outputfile << "skewness,";
		outputfile << "kurtosis,";
		outputfile << "meanlp,";
		outputfile << "maxlp,";
		outputfile << "minlp,";
		outputfile << "variancelp,";
		outputfile << "skewnesslp,";
		outputfile << "kurtosislp,";
		outputfile << "meanhp,";
		outputfile << "maxhp,";
		outputfile << "minhp,";
		outputfile << "variancehp,";
		outputfile << "skewnesshp,";
		outputfile << "kurtosishp,";

		outputfile << "num extrema,";
		outputfile << "num cycles,";
		outputfile << "damage,";
		outputfile << "meancycle,";
		outputfile << "maxcycle,";
		outputfile << "mincycle,";
		outputfile << "variancecycles,";
		outputfile << "skewnesscycles,";
		outputfile << "kurtosiscycles,";

		string prefix;
		prefix = "rawmax";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "rawmin";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "rawp2p";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "lpmax";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "lpmin";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "lpp2p";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "hpmax";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "hpmin";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";
		prefix = "hpp2p";
		outputfile << prefix << "numberofweibullamplitudes,";
		outputfile << prefix << "amplitudemean,";
		outputfile << prefix << "amplitudemax,";
		outputfile << prefix << "amplitudemin,";
		outputfile << prefix << "amplitudevariance,";
		outputfile << prefix << "amplitudeskew,";
		outputfile << prefix << "amplitudekurtosis,";
		outputfile << prefix << "lrbeta,";
		outputfile << prefix << "lryint,";
		outputfile << prefix << "lrcharval,";
		outputfile << prefix << "lrcorrelation,";
		outputfile << prefix << "mmbeta,";
		outputfile << prefix << "mmyint,";
		outputfile << prefix << "mmcharval,";

		outputfile << "psdpeakfrequency,";

		outputfile << "performrainflowanalysis,";
		outputfile << "performmmmanalysis,";
		outputfile << "performpsdanalysis,";
		outputfile << "performraoanalysis,";
		outputfile << "performweibullanalysis,";
		outputfile << "emptyscalevalue,";
		outputfile << "yellowlinevalue,";
		outputfile << "redlinevalue,";
		outputfile << "fullscalevalue,";
		outputfile << "piersidezerovalue,";
		outputfile << "channelinformation,";
		outputfile << "prescalefactorunits,";
		outputfile << "postscalefactorunits,";
		outputfile << "channellocation,";

		outputfile << "zerovarianceflag,";
		outputfile << "emptyscaleflag,";
		outputfile << "yellowlineflag,";
		outputfile << "redlineflag,";
		outputfile << "fullscaleflag,";
		outputfile << "hplpvarianceflag,";
		outputfile << "hplpmaxflag,";

		outputfile << "psd99pctfrequency,";

		outputfile << "modelscaleratio,";
		outputfile << "fullscalescanrate,";
		outputfile << "fullscaleseconds\n";

	}//end if

	for(int i=0; i < numchan; i++){

		outputfile << filename << ",";
		outputfile << numchan << ",";
		outputfile << virtualnumchan << ",";
		outputfile << scanrate << ",";
		outputfile << datechars;
		outputfile << analysistime << ",";
		outputfile << data[i].size() << ",";
		outputfile << data[i].size()/scanrate << ",";
		outputfile << version << ",";
		outputfile << fftsize << ",";
		outputfile << wavechannel << ",";
		outputfile << cutofffrequency << ",";

		outputfile << i << ",";
		outputfile << cals[i] << ",";
		outputfile << channelscalefactors[i] << ",";
		outputfile << channeloffsets[i] << ",";
		outputfile << channelnames[i] << ",";

		outputfile << means[i] << ",";
		outputfile << maxes[i] << ",";
		outputfile << mins[i] << ",";
		outputfile << variances[i] << ",";
		outputfile << skews[i] << ",";
		outputfile << kurtosi[i] << ",";

		outputfile << meanslp[i] << ",";
		outputfile << maxeslp[i] << ",";
		outputfile << minslp[i] << ",";
		outputfile << varianceslp[i] << ",";
		outputfile << skewslp[i] << ",";
		outputfile << kurtosilp[i] << ",";

		outputfile << meanshp[i] << ",";
		outputfile << maxeshp[i] << ",";
		outputfile << minshp[i] << ",";
		outputfile << varianceshp[i] << ",";
		outputfile << skewshp[i] << ",";
		outputfile << kurtosihp[i] << ",";

		outputfile << extrema[i].size() << ",";
		outputfile << rawcycles[i].size() << ",";
		outputfile << damage[i] << ",";
		outputfile << meanscycles[i] << ",";
		outputfile << maxescycles[i] << ",";
		outputfile << minscycles[i] << ",";
		outputfile << variancescycles[i] << ",";
		outputfile << skewscycles[i] << ",";
		outputfile << kurtosicycles[i] << ",";


		outputfile << weibullrawmax[i].amplitudes.size() << ",";
		outputfile << weibullrawmax[i].amplitudemean << ",";
		outputfile << weibullrawmax[i].amplitudemax << ",";
		outputfile << weibullrawmax[i].amplitudemin << ",";
		outputfile << weibullrawmax[i].amplitudevariance << ",";
		outputfile << weibullrawmax[i].amplitudeskew << ",";
		outputfile << weibullrawmax[i].amplitudekurtosis << ",";
		outputfile << weibullrawmax[i].lrbeta << ",";
		outputfile << weibullrawmax[i].lryint << ",";
		outputfile << weibullrawmax[i].lrcharval << ",";
		outputfile << weibullrawmax[i].lrcorrelation << ",";
		outputfile << weibullrawmax[i].mmbeta << ",";
		outputfile << weibullrawmax[i].mmyint << ",";
		outputfile << weibullrawmax[i].mmcharval << ",";

		outputfile << weibullrawmin[i].amplitudes.size() << ",";
		outputfile << weibullrawmin[i].amplitudemean << ",";
		outputfile << weibullrawmin[i].amplitudemax << ",";
		outputfile << weibullrawmin[i].amplitudemin << ",";
		outputfile << weibullrawmin[i].amplitudevariance << ",";
		outputfile << weibullrawmin[i].amplitudeskew << ",";
		outputfile << weibullrawmin[i].amplitudekurtosis << ",";
		outputfile << weibullrawmin[i].lrbeta << ",";
		outputfile << weibullrawmin[i].lryint << ",";
		outputfile << weibullrawmin[i].lrcharval << ",";
		outputfile << weibullrawmin[i].lrcorrelation << ",";
		outputfile << weibullrawmin[i].mmbeta << ",";
		outputfile << weibullrawmin[i].mmyint << ",";
		outputfile << weibullrawmin[i].mmcharval << ",";

		outputfile << weibullrawp2p[i].amplitudes.size() << ",";
		outputfile << weibullrawp2p[i].amplitudemean << ",";
		outputfile << weibullrawp2p[i].amplitudemax << ",";
		outputfile << weibullrawp2p[i].amplitudemin << ",";
		outputfile << weibullrawp2p[i].amplitudevariance << ",";
		outputfile << weibullrawp2p[i].amplitudeskew << ",";
		outputfile << weibullrawp2p[i].amplitudekurtosis << ",";
		outputfile << weibullrawp2p[i].lrbeta << ",";
		outputfile << weibullrawp2p[i].lryint << ",";
		outputfile << weibullrawp2p[i].lrcharval << ",";
		outputfile << weibullrawp2p[i].lrcorrelation << ",";
		outputfile << weibullrawp2p[i].mmbeta << ",";
		outputfile << weibullrawp2p[i].mmyint << ",";
		outputfile << weibullrawp2p[i].mmcharval << ",";

		outputfile << weibulllpmax[i].amplitudes.size() << ",";
		outputfile << weibulllpmax[i].amplitudemean << ",";
		outputfile << weibulllpmax[i].amplitudemax << ",";
		outputfile << weibulllpmax[i].amplitudemin << ",";
		outputfile << weibulllpmax[i].amplitudevariance << ",";
		outputfile << weibulllpmax[i].amplitudeskew << ",";
		outputfile << weibulllpmax[i].amplitudekurtosis << ",";
		outputfile << weibulllpmax[i].lrbeta << ",";
		outputfile << weibulllpmax[i].lryint << ",";
		outputfile << weibulllpmax[i].lrcharval << ",";
		outputfile << weibulllpmax[i].lrcorrelation << ",";
		outputfile << weibulllpmax[i].mmbeta << ",";
		outputfile << weibulllpmax[i].mmyint << ",";
		outputfile << weibulllpmax[i].mmcharval << ",";

		outputfile << weibulllpmin[i].amplitudes.size() << ",";
		outputfile << weibulllpmin[i].amplitudemean << ",";
		outputfile << weibulllpmin[i].amplitudemax << ",";
		outputfile << weibulllpmin[i].amplitudemin << ",";
		outputfile << weibulllpmin[i].amplitudevariance << ",";
		outputfile << weibulllpmin[i].amplitudeskew << ",";
		outputfile << weibulllpmin[i].amplitudekurtosis << ",";
		outputfile << weibulllpmin[i].lrbeta << ",";
		outputfile << weibulllpmin[i].lryint << ",";
		outputfile << weibulllpmin[i].lrcharval << ",";
		outputfile << weibulllpmin[i].lrcorrelation << ",";
		outputfile << weibulllpmin[i].mmbeta << ",";
		outputfile << weibulllpmin[i].mmyint << ",";
		outputfile << weibulllpmin[i].mmcharval << ",";

		outputfile << weibulllpp2p[i].amplitudes.size() << ",";
		outputfile << weibulllpp2p[i].amplitudemean << ",";
		outputfile << weibulllpp2p[i].amplitudemax << ",";
		outputfile << weibulllpp2p[i].amplitudemin << ",";
		outputfile << weibulllpp2p[i].amplitudevariance << ",";
		outputfile << weibulllpp2p[i].amplitudeskew << ",";
		outputfile << weibulllpp2p[i].amplitudekurtosis << ",";
		outputfile << weibulllpp2p[i].lrbeta << ",";
		outputfile << weibulllpp2p[i].lryint << ",";
		outputfile << weibulllpp2p[i].lrcharval << ",";
		outputfile << weibulllpp2p[i].lrcorrelation << ",";
		outputfile << weibulllpp2p[i].mmbeta << ",";
		outputfile << weibulllpp2p[i].mmyint << ",";
		outputfile << weibulllpp2p[i].mmcharval << ",";

		outputfile << weibullhpmax[i].amplitudes.size() << ",";
		outputfile << weibullhpmax[i].amplitudemean << ",";
		outputfile << weibullhpmax[i].amplitudemax << ",";
		outputfile << weibullhpmax[i].amplitudemin << ",";
		outputfile << weibullhpmax[i].amplitudevariance << ",";
		outputfile << weibullhpmax[i].amplitudeskew << ",";
		outputfile << weibullhpmax[i].amplitudekurtosis << ",";
		outputfile << weibullhpmax[i].lrbeta << ",";
		outputfile << weibullhpmax[i].lryint << ",";
		outputfile << weibullhpmax[i].lrcharval << ",";
		outputfile << weibullhpmax[i].lrcorrelation << ",";
		outputfile << weibullhpmax[i].mmbeta << ",";
		outputfile << weibullhpmax[i].mmyint << ",";
		outputfile << weibullhpmax[i].mmcharval << ",";

		outputfile << weibullhpmin[i].amplitudes.size() << ",";
		outputfile << weibullhpmin[i].amplitudemean << ",";
		outputfile << weibullhpmin[i].amplitudemax << ",";
		outputfile << weibullhpmin[i].amplitudemin << ",";
		outputfile << weibullhpmin[i].amplitudevariance << ",";
		outputfile << weibullhpmin[i].amplitudeskew << ",";
		outputfile << weibullhpmin[i].amplitudekurtosis << ",";
		outputfile << weibullhpmin[i].lrbeta << ",";
		outputfile << weibullhpmin[i].lryint << ",";
		outputfile << weibullhpmin[i].lrcharval << ",";
		outputfile << weibullhpmin[i].lrcorrelation << ",";
		outputfile << weibullhpmin[i].mmbeta << ",";
		outputfile << weibullhpmin[i].mmyint << ",";
		outputfile << weibullhpmin[i].mmcharval << ",";

		outputfile << weibullhpp2p[i].amplitudes.size() << ",";
		outputfile << weibullhpp2p[i].amplitudemean << ",";
		outputfile << weibullhpp2p[i].amplitudemax << ",";
		outputfile << weibullhpp2p[i].amplitudemin << ",";
		outputfile << weibullhpp2p[i].amplitudevariance << ",";
		outputfile << weibullhpp2p[i].amplitudeskew << ",";
		outputfile << weibullhpp2p[i].amplitudekurtosis << ",";
		outputfile << weibullhpp2p[i].lrbeta << ",";
		outputfile << weibullhpp2p[i].lryint << ",";
		outputfile << weibullhpp2p[i].lrcharval << ",";
		outputfile << weibullhpp2p[i].lrcorrelation << ",";
		outputfile << weibullhpp2p[i].mmbeta << ",";
		outputfile << weibullhpp2p[i].mmyint << ",";
		outputfile << weibullhpp2p[i].mmcharval << ",";

		outputfile << psdpeakfreqs[i] << ",";

		outputfile << performrainflowanalysis[i] << ",";
		outputfile << performmmmanalysis[i] << ",";
		outputfile << performpsdanalysis[i] << ",";
		outputfile << performraoanalysis[i] << ",";
		outputfile << performweibullanalysis[i] << ",";
		outputfile << emptyscalevalues[i] << ",";
		outputfile << yellowlinevalues[i] << ",";
		outputfile << redlinevalues[i] << ",";
		outputfile << fullscalevalues[i] << ",";
		outputfile << piersidezerovalues[i] << ",";
		outputfile << channelinformation[i] << ",";
		outputfile << prescalefactorunits[i] << ",";
		outputfile << postscalefactorunits[i] << ",";
		outputfile << channellocations[i] << ",";

		outputfile << zerovarianceflag[i] << ",";
		outputfile << emptyscaleflag[i] << ",";
		outputfile << yellowlineflag[i] << ",";
		outputfile << redlineflag[i] << ",";
		outputfile << fullscaleflag[i] << ",";
		outputfile << hplpvarianceflag[i] << ",";
		outputfile << hplpmaxflag[i] << ",";

		outputfile << psd99pctfreqs[i] << ",";

		outputfile << modelscaleratio << ",";
		outputfile << fullscanrate << ",";
		outputfile << data[i].size()/fullscanrate << "\n";

	}//end for

	outputfile.close();

}//end outputstatsreport


//Fast Fourier Transform
//straight out of numerical recipes in C Second Edition 12.2 p. 504
//vector size (nn) must be an integer power of 2, which is the number of real or complex data points
//computes discrete Fourier transform if isign = 1
//computes inverse discrete Fourier transform if isign = -1
//resulting ft real and imaginary parts are each sized nn and include positive and negative frequencies
//resulting ft starts at freq 0|+DeltaF...+nyquist|-nyquist...-DeltaF
void fft(float realarray[], float imaginaryarray[], unsigned long nn, int isign){
	unsigned long n,mmax,m,j,istep,i;
	double wtemp,wr,wpr,wpi,wi,theta;
	float tempr,tempi;

	//convert seperate real array and imaginary array to interleaved complex format in book
	float* da = new float[nn*2+1];
	for(i=0; i < nn*2+1;i++){
		da[i] = 0;
	}//end for
	j=1;
	for(i=0; i < nn; i++){
		da[j] = realarray[i];
		da[j+1] = imaginaryarray[i];
		j+=2;
	}//end for

	n=nn << 1;//short for multiply by 2
	j=1;
	for(i=1;i<n;i+=2){
		if(j>i){
			SWAP(da[j],da[i]);
			SWAP(da[j+1],da[i+1]);
		}//end if
		m=n >> 1;
		while(m >= 2 && j > m){
			j -= m;
			m >>= 1;
		}//end while
		j += m;
	}//end for

	mmax=2;
	while( n > mmax){
		istep=mmax << 1;
		theta=isign*(6.28318530717959/mmax);
		wtemp=sin(0.5*theta);
		wpr = -2.0*wtemp*wtemp;
		wpi = sin(theta);
		wr=1.0;
		wi=0.0;
		for(m=1;m<mmax;m+=2){
			for(i=m;i<=n;i+=istep){
				j=i+mmax;
				tempr=wr*da[j]-wi*da[j+1];
				tempi=wr*da[j+1]+wi*da[j];
				da[j]= da[i]-tempr;
				da[j+1]= da[i+1]-tempi;
				da[i] += tempr;
				da[i+1] += tempi;
			}//end for
			wr=(wtemp=wr)*wpr-wi*wpi+wr;
			wi=wi*wpr+wtemp*wpi+wi;
		}//end for
		mmax = istep;
	}//end while

	//convert interleaved complex format back to real array and imaginary array
	j=1;
	for(i=0; i < nn; i++){
		realarray[i] = da[j];
		imaginaryarray[i] = da[j+1];
		j+=2;
	}//end for

	delete da;
}//end fft


//this function computes the Power Spectral Density of a selected channel using
//the finite Fourier transform on numerous overlapping
//segments of the input data file. The psds from the different semgments are averaged together
//to create an average power spectra over the entire data file.
//This function uses the simple power spectra defined in Numerical Recipes in C p. 498
//P(f) = |H(f)|^2 + |H(-f)|^2 = 2|H(f)|^2
void computepsd(int channel){

	float* ftr = new float[fftsize];//temporarily holds the fourier transform of a psd segment; real component
	float* fti = new float[fftsize];//temporarily holds the fourier transform of a psd segment; imaginary component
	double* psd = new double[fftsize/2];//temporarily holds the average of the segment ffts

	unsigned int i=0;
	for(i=0; i < psds.size(); i++){
		psds[i].resize(fftsize/2);
	}//end for

	for(i=0; i < fftsize/2; i++){
		psd[i] = 0.0;
	}//end for

	double segmentmean = 0;

	unsigned int segment = 0;
	unsigned int datachannelsize = data[channel].size();
	//loop through segments and average them with 50% overlap
	while( ((segment+2) * fftsize/2) < datachannelsize && i < datachannelsize){

		//copy the segment of data 50% overlap
		for(i=0; i < fftsize; i++){
			ftr[i] = data[channel][i+segment*fftsize/2];
			fti[i] = 0;//no imaginary component for real data
		}//end for

		//subtract the mean of each segment to prevent ultra-low frequencies from aliasing to DC
		segmentmean =0;
		for(i=0; i < fftsize; i++){
			segmentmean += ftr[i];
		}//end for
		segmentmean /= fftsize;
		for(i=0; i < fftsize; i++){
			ftr[i] -= (float)segmentmean;
		}//end for

		//perform fft on the segment
		//resulting ft is fftsize including positive and negative frequencies
		fft(ftr,fti,fftsize,1);

		//add the real component of the ft to the psd average
		//take just the positive frequencies
		//division by (fftsize*scanrate/2) is a necessary scale factor
		for(i = 0; i < fftsize/2; i++){
			psd[i] += (double)2.0*ABS(ftr[i])*ABS(ftr[i])/(fftsize*fullscanrate/2);
		}//end for

		segment++;
	}//end while

	//divide by the number of segments to get the average psd
	for(i=0; i < fftsize/2; i++){
		psd[i] /= (double)segment;
	}//end for

	//store the psd
	for(i=0; i < fftsize/2; i++){
		psds[channel][i] = (float)psd[i];
	}//end for

	if(ftr != NULL) delete ftr;
	if(fti != NULL) delete fti;
	if(psd != NULL) delete psd;

	//compute the psd peak frequency index
	float peakmagnitude = psds[channel][0];
	psdpeakfreqs[channel] = 0;
	for(i=1; i < psds[channel].size(); i++){
		if(psds[channel][i] > peakmagnitude){
			peakmagnitude = psds[channel][i];
			psdpeakfreqs[channel] = (float)i;
		}//end if
	}//end for


	//compute the psd 99pct energy frequency index
	double integral = 0;
	psd99pctfreqs[channel] = 0;
	for(i=0; i < psds[channel].size(); i++){
		integral += psds[channel][i];
	}//end for
	double psd99pctvalue = integral * 0.99;
	integral = 0;
	for(i=0; i < psds[channel].size() && integral < psd99pctvalue; i++){
		integral += psds[channel][i];
		psd99pctfreqs[channel] = (float)i;
	}//end for

}//end computepsd


//This function outputs the psds for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output psd file name is the same as the input file name
//with -psd added between the name and extension.
//Example: input: run_77.scl outupt: run_77-psd.scl
//The resulting scl file can be loaded directly into IGOR.
void outputpsdreportbinary(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -psd added
	string fname = filename;
	fname[fname.size()-4] = '-';
	fname[fname.size()-3] = 'p';
	fname[fname.size()-2] = 's';
	fname[fname.size()-1] = 'd';
	fname.append(".scl");

	//open and overwrite any existing binary output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc|ios::binary);

	//scl header number of channels
	int tempnumchan = swapbytes(numchan);
	outputfile.write((char*)&tempnumchan,4);

	//scl header scan rate
	float tempdeltat = fullscanrate/fftsize;
	float tempscanrate = swapbytes((float)(1/tempdeltat));
	outputfile.write((char*)&tempscanrate,4);

	//convert psd peak frequency and psd 99pct frequency index to hz
	for(i=0; i < (unsigned int)numchan; i++){
		psdpeakfreqs[i] = psdpeakfreqs[i] * tempdeltat;
		psd99pctfreqs[i] = psd99pctfreqs[i] * tempdeltat;
	}//end for

	//scl header cal factors identical to input data file
	double tempcal;
	for(i=0; i < (unsigned int)numchan; i++){
		tempcal = swapbytes(cals[i]);
		outputfile.write((char*)&tempcal,8);
	}//end for

	//scl interleaved data (actual psds)
	for(i=0; i < fftsize/2; i++){
		for(j=0; j < (unsigned int)numchan; j++){
			datapt = swapbytes(psds[j][i]);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputpsdreport


//this function computes the Response Amplitude Operator of a selected channel using
//by dividing its PSD by the PSD of the wave height channel
void computerao(int channel){

	//initialize rao memory storage
	unsigned int i=0;
	for(i=0; i < raos.size(); i++){
		raos[i].resize(fftsize/2);
	}//end for

	//compute the RAO by dividing the channel PSD by the wave height PSD and taking the square root
	raos[channel][0] = 0;
	for(i=1; i < fftsize/2; i++){
		raos[channel][i] = sqrt(psds[channel][i]/psds[wavechannel][i]);
	}//end for
}//end computerao


//This function outputs the raos for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output rao file name is the same as the input file name
//with -rao added between the name and extension.
//Example: input: run_77.scl outupt: run_77-rao.scl
//The resulting scl file can be loaded directly into IGOR.
void outputraoreportbinary(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -rap added
	string fname = filename;
	fname[fname.size()-4] = '-';
	fname[fname.size()-3] = 'r';
	fname[fname.size()-2] = 'a';
	fname[fname.size()-1] = 'o';
	fname.append(".scl");

	//open and overwrite any existing binary output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc|ios::binary);

	//scl header number of channels
	int tempnumchan = swapbytes(numchan);
	outputfile.write((char*)&tempnumchan,4);

	//scl header scan rate
	float tempdeltat = fullscanrate/fftsize;
	float tempscanrate = swapbytes((float)(1/tempdeltat));
	outputfile.write((char*)&tempscanrate,4);

	//scl header cal factors identical to input data file
	double tempcal;
	for(i=0; i < (unsigned int)numchan; i++){
		tempcal = swapbytes(cals[i]);
		outputfile.write((char*)&tempcal,8);
	}//end for

	//scl interleaved data (actual psds)
	for(i=0; i < fftsize/2; i++){
		for(j=0; j < (unsigned int)numchan; j++){
			datapt = swapbytes(raos[j][i]);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputraoreport


//This function performs Optimal "Wiener" Filtering
//as described in Numberical Recipes in C Second Edition 13.3 p547 and 13.5 p558
//Lowpass only; frequencies above the cutofffrequency are removed.
//The filtered time history that represents the highpass should be created through raw-lp
//subtraction because that is order n whereas the fftfilter is 2*n*log2(n).
void fftfilterlp(int channel){
	datalp[channel].resize(data[channel].size());

	//compute the next power of 2
	originaldatasize = data[channel].size();
	unsigned int i = 0;
	while(powerof2 < originaldatasize){
		powerof2 = (unsigned int)pow(2.0, (int)i);
		i++;
	}//end while
	i--;

	//compute the mean to subtract before filtering
	//the mean will have already been computed if raw mmm analysis was performed before this function call
	//so means[channel] can be used instead of segmentmean computed here to save some calculations
	/*
	double segmentmean;
	segmentmean =0;
	for(i=0; i < originaldatasize; i++){
		segmentmean += data[channel][i];
	}//end for
	segmentmean /= originaldatasize;
	*/

	//pad the data to the next power of 2 and subtract the computed mean
	if(paddedrealdata == NULL) paddedrealdata = new float[powerof2];
	if(paddedimaginarydata == NULL) paddedimaginarydata = new float[powerof2];
	for(i=0; i < originaldatasize; i++){
		paddedrealdata[i] = data[channel][i] - (float)means[channel];
		paddedimaginarydata[i] = 0;
	}//end for
	for(i=originaldatasize; i < powerof2; i++){
		paddedrealdata[i] = 0;
		paddedimaginarydata[i] = 0;
	}//end for

	//compute the fft
	fft(paddedrealdata,paddedimaginarydata,powerof2,1);

	//apply the filter function... in this case remove undesired frequencies
	float deltaf = fullscanrate/powerof2;
	unsigned int cutoffindex = (unsigned int)(cutofffrequency/deltaf);
	for(i = cutoffindex; i < powerof2-cutoffindex; i++){
		paddedrealdata[i] = 0;
		paddedimaginarydata[i] = 0;
	}//end for

	//compute the inverse fft
	fft(paddedrealdata,paddedimaginarydata,powerof2,-1);

	//store the filtered result and add the mean back in
	for(i=0; i < originaldatasize; i++){
		datalp[channel][i] = paddedrealdata[i]/powerof2 + (float)means[channel];
	}//end for

}//end fftfilterlp


//This function computes the difference between the original
//raw input data and the lowpass filtered version, which
//yields the highpass version. This algorithm is order n, which saves
//calculations over the fftfilter 2*n*log2(n) for creating the lowpass version.
void difffilterhp(int channel){
	datahp[channel].resize(data[channel].size());
	for(unsigned int i=0; i < data[channel].size(); i++){
		datahp[channel][i] = data[channel][i] - datalp[channel][i];
	}//end for
}//end difffilterhp


//This function finds amplitudes given a set of mean crossings for Weibull analysis.
void findamplitudes(int channel, vector<vector<float> >&data, vector<weibullstorage>& weibullmax, vector<weibullstorage>& weibullmin, vector<weibullstorage>& weibullp2p, vector<unsigned int>& lpmeancrossings){
	float max;
	float min;
	//step through mean crossings to find amplitudes
	for(unsigned int i=0; i < lpmeancrossings.size()-2; i+=2){
		max = data[channel][lpmeancrossings[i]];
		min = data[channel][lpmeancrossings[i]];
		for(unsigned int j=lpmeancrossings[i]; j <= lpmeancrossings[i+2]; j++){
			if(data[channel][j] > max){
				max = data[channel][j];
			}//end if
			else if(data[channel][j] < min){
				min = data[channel][j];
			}//end if
		}//end for
		weibullmax[channel].amplitudes.insert(weibullmax[channel].amplitudes.end(),ABS(max));
		weibullmin[channel].amplitudes.insert(weibullmin[channel].amplitudes.end(),ABS(min));
		weibullp2p[channel].amplitudes.insert(weibullp2p[channel].amplitudes.end(),max-min);
	}//end for
}//end findamplitudes


double DetermineXFromCDF2ParameterWeibull(double P, double shape, double scale){
	double X;
	X = scale*pow(-log(1-P),1/shape);
	//if(P == 0.99)cout << "P: " << P << " B: " << shape << " C: " << scale << " X: " << X << endl;
	return X;
}//end DetermineXFromCDF2ParameterWeibull


void LinearRegressionLeastSquares4Weibull(int channel, vector<weibullstorage>& weibull){
	if(weibull[channel].amplitudes.size() != weibull[channel].rank.size()) return;
	if(weibull[channel].amplitudes.size() <= 1 || weibull[channel].amplitudevariance == 0){
		weibull[channel].lryint = 0;
		weibull[channel].lrbeta = 0;
		weibull[channel].lrcorrelation = 0;
		weibull[channel].lrcharval = 0;
		return;
	}//end if

	double sumx = 0;
	double sumxsq = 0;
	double sumy = 0;
	double sumysq = 0;
	double sumxy = 0;
	//ymean = 0;

	for(unsigned int i=0; i < weibull[channel].rank.size(); i++){
		sumx += weibull[channel].rank[i];
		sumxsq += weibull[channel].rank[i]*weibull[channel].rank[i];
		sumy += weibull[channel].amplitudes[i];
		sumysq += weibull[channel].amplitudes[i] * weibull[channel].amplitudes[i];
		sumxy += weibull[channel].rank[i] * weibull[channel].amplitudes[i];
	}//end for

	//ymean = sumy/weibull[channel].amplitudes.size();

	weibull[channel].lryint = (sumy*sumxsq - sumx*sumxy)/((double)weibull[channel].amplitudes.size()*sumxsq - sumx*sumx);

	weibull[channel].lrbeta = ((double)weibull[channel].amplitudes.size()*sumxy - sumx*sumy)/((double)weibull[channel].amplitudes.size()*sumxsq - sumx*sumx);

	weibull[channel].lrcorrelation = ((double)weibull[channel].amplitudes.size()*sumxy - sumx*sumy)/sqrt(((double)weibull[channel].amplitudes.size()*sumxsq - sumx*sumx)*((double)weibull[channel].amplitudes.size()*sumysq - sumy*sumy));
	weibull[channel].lrcorrelation *= weibull[channel].lrcorrelation;

	weibull[channel].lrcharval = exp(-weibull[channel].lryint/weibull[channel].lrbeta);

	weibull[channel].lrcdf99 = DetermineXFromCDF2ParameterWeibull(0.99,weibull[channel].lrbeta,weibull[channel].lrcharval);
	weibull[channel].lrcdf999 = DetermineXFromCDF2ParameterWeibull(0.999,weibull[channel].lrbeta,weibull[channel].lrcharval);
	weibull[channel].lrcdf9999 = DetermineXFromCDF2ParameterWeibull(0.9999,weibull[channel].lrbeta,weibull[channel].lrcharval);
}//end Linear Regression Least Squares Method 4 Weibull


//returns the natural log of the gamma function
//straight out of Numerical Recipes in C Second Edition
//Section 6.0 page 214
float lngamma(float xx){
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,24.01409824083091,-1.231739572450155,0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for(j=0;j<=5;j++) ser += cof[j]/++y;
	return (float)(-tmp+log(2.5066282746310005*ser/x));
}//end lngamma
double lngamma(double xx){
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,24.01409824083091,-1.231739572450155,0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for(j=0;j<=5;j++) ser += cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}//end lngamma


//computes the gamma function
float Gamma(float xx){
	return exp(lngamma(xx));
}//end Gamma
double Gamma(double xx){
	return exp(lngamma(xx));
}//end Gamma


void momentmethod2parameter(int channel, vector<weibullstorage>& weibull){
	if(weibull[channel].amplitudes.size() != weibull[channel].rank.size()) return;
	if(weibull[channel].amplitudes.size() <= 1 || weibull[channel].amplitudevariance == 0){
		weibull[channel].mmbeta = 0;
		weibull[channel].mmcharval = 0;
		weibull[channel].mmyint = 0;
		return;
	}//end if

	double VarianceDivMeanSq;
	double lb = 0.05;
	double ub = 5.00;
	double bs = 0.001;

	VarianceDivMeanSq = weibull[channel].amplitudevariance/(weibull[channel].amplitudemean*weibull[channel].amplitudemean);
	weibull[channel].mmbeta = lb;//defined start/init value
	for(double b=lb; b <= ub; b+=bs){
			if( ABS((Gamma(1+2/b)/(Gamma(1+1/b)*Gamma(1+1/b)) - 1) - VarianceDivMeanSq) <
					ABS((Gamma(1+2/weibull[channel].mmbeta )/(Gamma(1+1/weibull[channel].mmbeta)*Gamma(1+1/weibull[channel].mmbeta)) - 1) - VarianceDivMeanSq) ){
				weibull[channel].mmbeta = b;
			}//end if
	}//end for

	//calculate the moment method char val
	//and y intercepts
	weibull[channel].mmcharval = weibull[channel].amplitudemean/Gamma(1+1/weibull[channel].mmbeta);
	weibull[channel].mmyint = -weibull[channel].mmbeta*log(ABS(weibull[channel].mmcharval));

	weibull[channel].mmcdf99 = DetermineXFromCDF2ParameterWeibull(0.99,weibull[channel].mmbeta,weibull[channel].mmcharval);
	weibull[channel].mmcdf999 = DetermineXFromCDF2ParameterWeibull(0.999,weibull[channel].mmbeta,weibull[channel].mmcharval);
	weibull[channel].mmcdf9999 = DetermineXFromCDF2ParameterWeibull(0.9999,weibull[channel].mmbeta,weibull[channel].mmcharval);

}//end momentmethod2parameter


//This function calculates the 2-parameter statistics on a set of
//amplitudes.
void weibullstats(int channel, vector<weibullstorage>& weibull){
	//compute amplitude stats
	mmmstats(weibull[channel].amplitudes,weibull[channel].amplitudemean,weibull[channel].amplitudemax,weibull[channel].amplitudemin,weibull[channel].amplitudevariance,weibull[channel].amplitudeskew,weibull[channel].amplitudekurtosis);

	//sort amplitudes
	quicksort(weibull[channel].amplitudes);

	//determine rank
	for(unsigned int i=0; i < weibull[channel].amplitudes.size(); i++){
		weibull[channel].rank.insert(weibull[channel].rank.end(),(float)i+1);
	}//end for

	//linearize data in Weibull space
	for(unsigned int i=0; i < weibull[channel].rank.size(); i++){
		weibull[channel].rank[i] = log( weibull[channel].amplitudes[i] );
		weibull[channel].amplitudes[i] = log( log( 1/(1-((i+1)/((float)weibull[channel].rank.size()+1)) ) ) );
	}//end for

	//perform linear regression and calculate parameters
	LinearRegressionLeastSquares4Weibull(channel, weibull);

	//calculate moment method parameters
	momentmethod2parameter(channel,weibull);
}//end weibullstats


//This function performs 2-parameter linear regression and
//2-parameter moment method Weibull analysis as described in
//NSWCCD-65-TR2005/17 High Volume Data Analysis Suite
//The data shoud have already been filtered by this point.
void weibullanalysis(int channel){
	//subtract mean
	for(unsigned int i=0; i < data[channel].size(); i++){
		data[channel][i] -= (float)means[channel];
		if(cutofffrequency != 0){
			datalp[channel][i] -= (float)meanslp[channel];
			datahp[channel][i] -= (float)meanshp[channel];
		}//end if
	}//end for

	//store mean crossings in lp filtered version
	vector<unsigned int> lpmeancrossings;
	if(cutofffrequency != 0){
		//add first point
		lpmeancrossings.insert(lpmeancrossings.end(),0);
		for(unsigned int i=1; i < datalp[channel].size()-1 && lpmeancrossings.size() < datalp[channel].size()/10; i++){
			if((datalp[channel][i] > 0 && datalp[channel][i+1] <= 0) ||(datalp[channel][i] < 0 && datalp[channel][i+1] >= 0)){
				lpmeancrossings.insert(lpmeancrossings.end(),i+1);
			}//end if
		}//end for
		//add last point
		lpmeancrossings.insert(lpmeancrossings.end(),datalp[channel].size()-1);
	}//end if
	else{
		//add first point
		lpmeancrossings.insert(lpmeancrossings.end(),0);
		for(unsigned int i=1; i < data[channel].size()-1 && lpmeancrossings.size() < data[channel].size()/10; i++){
			if((data[channel][i] > 0 && data[channel][i+1] <= 0) ||(data[channel][i] < 0 && data[channel][i+1] >= 0)){
				lpmeancrossings.insert(lpmeancrossings.end(),i+1);
			}//end if
		}//end for
		//add last point
		lpmeancrossings.insert(lpmeancrossings.end(),data[channel].size()-1);
	}//end else

	if(lpmeancrossings.size() < 3) return;

	//detect amplitudes
	findamplitudes(channel, data, weibullrawmax, weibullrawmin, weibullrawp2p, lpmeancrossings);
	if(cutofffrequency != 0){
		findamplitudes(channel, datalp, weibulllpmax, weibulllpmin, weibulllpp2p, lpmeancrossings);
		findamplitudes(channel, datahp, weibullhpmax, weibullhpmin, weibullhpp2p, lpmeancrossings);
	}//end if

	//calculate weibull statistics
	weibullstats(channel, weibullrawmax);
	weibullstats(channel, weibullrawmin);
	weibullstats(channel, weibullrawp2p);
	if(cutofffrequency != 0){
		weibullstats(channel, weibulllpmax);
		weibullstats(channel, weibulllpmin);
		weibullstats(channel, weibulllpp2p);
		weibullstats(channel, weibullhpmax);
		weibullstats(channel, weibullhpmin);
		weibullstats(channel, weibullhpp2p);
	}//end if
}//end weibullanalysis


//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "version:\t" << version << endl;
	cout << "starttime:\t" << starttime << endl;

	//load input data file
	cout << "filename:\t" << argv[1] << endl;
	cout << "reading time histories: "; cout.flush();
	loadSCLfile(argv[1]);
	resizeVectors();
	cout << "done\n"; cout.flush();
	cout << "number of channels:\t" << numchan << endl;
	cout << "scan rate:\t" << scanrate << endl;
	cout << "total scans:\t" << data[0].size() << endl;
	cout << "seconds:\t" << data[0].size()/scanrate << endl;

	//load config file
	cout << "trying to load config file: "; cout.flush();
	tryToLoadConfigFile();
	cout << "done\n"; cout.flush();

	//more outputs
	cout << "model scale ratio:\t" << modelscaleratio << endl;
	cout << "full scale scan rate:\t" << fullscanrate << endl;
	cout << "full scale seconds:\t" << data[0].size()/fullscanrate << endl;

	//apply scale factors and offsets
	cout << "applying scale factors and offsets: "; cout.flush();
	applysfo();
	cout << "done\n"; cout.flush();

	//compute virtual channels
	cout << "computing virtual channels: "; cout.flush();
	computevirtualchannels();
	cout << "done\n"; cout.flush();
	cout << "number of virtual channels:\t" << virtualnumchan << endl;

	//compute rainflow statistics
	cout << "computing rainflow damage:"; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performrainflowanalysis[i]){
			selectrainflowextrema(i);//selects extrema from raw data
			performrainflow(i);//performs rainflow cycle counting
			applyMinersRule(i);//applies Miner's Rule to accumulate damage for the input data file
			mmmstats(rawcycles[i],meanscycles[i],maxescycles[i],minscycles[i],variancescycles[i],skewscycles[i],kurtosicycles[i]);//computes mmm stats on detected cycles
			computefullcyclehistogram(i);//compute histogram of full cycles
			cout << "."; cout.flush();
		}//end if
	}//end for
	outputfullcyclehistogram(argv[1]);//outpus histogram to an scl file
	cout << "done\n"; cout.flush();

	//compute raw mmm statistics
	cout << "computing raw mmm statistics: "; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performmmmanalysis[i] || performweibullanalysis[i]){
			mmmstats(data[i],means[i],maxes[i],mins[i],variances[i],skews[i],kurtosi[i]);
			cout << "."; cout.flush();
		}//end if
	}//end for
	cout << "done\n"; cout.flush();

	//compute filtered mmm statistics... uses raw means so this must come after raw stats
	if(cutofffrequency != 0){
		//compute lowpass mmm statistics
		cout << "computing lowpass filter and mmm statistics: "; cout.flush();
		for(int i=0; i < numchan; i++){
			if(performmmmanalysis[i]){
				fftfilterlp(i);//2*n*log2(n)
				mmmstats(datalp[i],meanslp[i],maxeslp[i],minslp[i],varianceslp[i],skewslp[i],kurtosilp[i]);
				cout << "."; cout.flush();
			}//end if
		}//end for
		if(paddedrealdata != NULL)delete paddedrealdata;//populated during fftfilterlp
		if(paddedimaginarydata != NULL)delete paddedimaginarydata;//populated during fftfilterlp
		cout << "done\n"; cout.flush();

		//compute highpass mmm statistics
		cout << "computing highpass filter and mmm statistics: "; cout.flush();
		for(int i=0; i < numchan; i++){
			if(performmmmanalysis[i]){
				difffilterhp(i);//n
				mmmstats(datahp[i],meanshp[i],maxeshp[i],minshp[i],varianceshp[i],skewshp[i],kurtosihp[i]);
				cout << "."; cout.flush();
			}//end if
		}//end for
		cout << "done\n"; cout.flush();
	}//end if

	//compute flags from statistics
	cout << "computing flags: "; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performmmmanalysis[i]){
			computeflags(i);
			cout << "."; cout.flush();
		}//end if
	}//end for
	cout << "done\n"; cout.flush();

	//compute psd statistics
	cout << "computing psd statistics: "; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performpsdanalysis[i] || performraoanalysis[i]){
			computepsd(i);
			cout << "."; cout.flush();
		}//end if
	}//end for
	outputpsdreportbinary(argv[1]);//outpus psds to an scl file
	cout << "done\n"; cout.flush();

	//compute rao statistics
	cout << "computing rao statistics: "; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performraoanalysis[i]){
			computerao(i);
			cout << "."; cout.flush();
		}//end if
	}//end for
	outputraoreportbinary(argv[1]);//outpus raos to an scl file
	cout << "done\n"; cout.flush();

	//compute Weibull statistics
	cout << "computing Weibull statistics: "; cout.flush();
	for(int i=0; i < numchan; i++){
		if(performweibullanalysis[i]){
			weibullanalysis(i);
			cout << "."; cout.flush();
		}//end if
	}//end for
	cout << "done\n"; cout.flush();

	//output rainflow, raw mmm, lp mmm, hp mmm, weibull, and psd peak
	cout << "writing to statslog.csv: "; cout.flush();
	outputstatsreport(argv[1]);//outupt includes mmm stats and rainflow stats and weibull stats and psd stats
	cout << "done\n"; cout.flush();

	//calculate accumulated damage
	cout << "calculating accumulated damage: "; cout.flush();
	accumulatedamagereportbinary();//alters what is stored in damage vector, so this must come after statslog output
	accumulatedamagereportascii();//outputs an ascii version of the accumulated damage
	cout << "done\n"; cout.flush();

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;
	cout << "daq time / analysis time:\t" << data[0].size()/scanrate/(endtime-starttime) << endl;
	cout.flush();

	//calls external script/batch file
	//could zip up the output files or does anything else you want
	//it to do after analyzing a single data file
	system("ae-postanalysis.bat");

	return 0;//exit cleanly
}//end main


