/* ascii2scl.cpp - lastupdate 200612041222
Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com

 Converts ascii comma or tab delimited data files to scl binary data files
*/

//include all the good functions and variables from ae
#include"../../ae/ae/ae.h"


//global variables
int headersize = 3;//number of ascii lines to skip
int filenumber = 0;//current file number
int originalnumchan = 0;//number of channels for the first file


//getline function that detects comma or tab
string getline_multiple(ifstream& in){
	char ch;
	string res;
	while (in.get(ch) && ch != ',' && ch != '\t' && ch != '\n' && !in.eof())
		res += ch;
	return res;
}//end getline_multiple


//This function loads a standard ASCII comma or tab delimited data file
void loadASCIIfile(string filename){
	ifstream inputfile;
	inputfile.open(filename.data(), ios::binary);

	if(!inputfile.good()){
		cout << "Error: Cannot read input file " << filename << ".\n";
		cerr << "Error: Cannot read input file " << filename << ".\n";
		inputfile.close();
		exit(1);
	}//end if

	//skip header
	char* charray = new char[8192];
	for(int i=0; i < headersize; i++){
		inputfile.getline(charray,8192);
	}//end for

	//read number of channels from first data line
	unsigned int startpos = inputfile.tellg();
	char peekchar = ' ';
	char discardchar = ' ';
	numchan = 0;
	while(peekchar != '\n'){
		peekchar = inputfile.peek();
		if(peekchar == ',' || peekchar == '\t' || peekchar == '\n')
			numchan++;
		discardchar = inputfile.get();
	}//end while
	inputfile.seekg(startpos);

	data.resize(numchan);

	//read all cal factors
	cals.resize(numchan);
	for(int i=0; i<numchan; i++){
		cals[i] = 1.0;
	}//end for

	string datastring;

	//ready to read interleaved data at this point using numchan until end of file
	while(!inputfile.eof()){
		for(int i=0; i<numchan && !inputfile.eof(); i++){
			datastring = getline_multiple(inputfile);
			if(!inputfile.eof()){
				data[i].insert(data[i].end(),(float)atof(datastring.data()));
			}//end if
		}//end for
	}//end while

	inputfile.close();
}//end loadASCIIfile


//This function outputs the data for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output psd file name is the same as the input file name
//with the extension changed to .scl.
//Example: input: run_77.dat outupt: run_77.scl
//The resulting scl file can be loaded directly into IGOR.
void outputSCLfile(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -psd added
	string fname = filename;
	fname[fname.size()-3] = 's';
	fname[fname.size()-2] = 'c';
	fname[fname.size()-1] = 'l';

	//open and overwrite any existing binary output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc|ios::binary);

	//scl header number of channels
	int tempnumchan = swapbytes(numchan);
	outputfile.write((char*)&tempnumchan,4);

	//scl header scan rate
	float tempscanrate = swapbytes(scanrate);
	outputfile.write((char*)&tempscanrate,4);

	//scl header cal factors identical to input data file
	double tempcal;
	for(i=0; i < (unsigned int)numchan; i++){
		tempcal = swapbytes(cals[i]);
		outputfile.write((char*)&tempcal,8);
	}//end for

	//scl interleaved data
	for(i=0; i < data[0].size(); i++){
		for(j=0; j < (unsigned int)numchan; j++){
			datapt = swapbytes(data[j][i]);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputSCLfile


//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "starttime:\t" << starttime << endl;

	//cycle through input files
	for(filenumber=0; filenumber < argc-1; filenumber++){
		//load input data file
		cout << "filename:\t" << argv[filenumber+1] << endl;
		cout << "reading time histories: ";
		loadASCIIfile(argv[filenumber+1]);
		if(filenumber ==0) originalnumchan = numchan;
		else if( numchan != originalnumchan) cout << "Error: numchan does not match originalnumchan\n";
		cout << "done\n";
		cout << "number of channels:\t" << numchan << endl;
		scanrate = 1;
		cout << "scan rate:\t" << scanrate << endl;
		cout << "total scans:\t" << data[0].size() << endl;
		cout << "seconds:\t" << data[0].size()/scanrate << endl;

		//write output file
		cout << "writing output file: ";
		outputSCLfile(argv[filenumber+1]);
		cout << "done\n";

		//clean up
		for(int i=0;i < numchan; i++){
			data[i].resize(0);//clear arrays if this is not the first file
		}//end for
		data.resize(0);
		cals.resize(0);
	}//end for

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;

	return 0;
}//end main


