
#include "guitest-fluid.h" //header file automatically created by fluid gui designer
#include "guitest.h" //common header file accompanying this cpp file



/*
Dump of file guitest.exe

File Type: EXECUTABLE IMAGE

  Image has the following dependencies:

    WSOCK32.dll
    COMCTL32.dll
    KERNEL32.dll
    USER32.dll
    GDI32.dll
    SHELL32.dll
    ole32.dll
    MSVCP80.dll
    MSVCR80.dll
    msvcm80.dll
    mscoree.dll
*/







/*
necessary msvc project options to build with FLTK
msvc-c++-general-include path: C:\Documents and Settings\Owner\Desktop\fltk-1.1.7
msvc-linker-lib path: C:\Documents and Settings\Owner\Desktop\fltk-1.1.7\lib
msvc-linker-input libs fltkd.lib fltkformsd.lib fltkgld.lib fltkimagesd.lib fltkjpegd.lib fltkpngd.lib fltkzd.lib wsock32.lib comctl32.lib
*/







/*
system path requirements
wgnuplot
acrord32
*/






/*
here are the functions used directly from other AE programs
ae - void resizeVectors(void)
ae - void tryToLoadConfigFile(void)
slae - class statslogline
slae - void statslogline::readline(ifstream& inputfile)
slae - void readstatslog(string filename)
slae - void determinescanratesandchannels(void)
ae - int swapbytes(int input)
ae - float swapbytes(float input)
ae - double swapbytes(double input)
ae - void loadSCLfile(string filename)
*/




//the function loads the desired aeconfig-numchan-scanrate.csv file
//and resizes vectors
void loadaeconfigcsv(void){
	infile.clear();

	//build proper filename
	string filename = "aeconfig-";
	char temp[1024];
	sprintf(temp,"%i",realnumchan);
	filename.append(temp);
	filename.append("-");
	sprintf(temp,"%i",(int)scanrate);
	filename.append(temp);
	filename.append(".csv");
	cout << "loading " << filename << " file:";
	logtimestamp();
	logoutput("loading ");
	logoutput(filename);
	logoutput(" file:");

	//reset numchan, virtualnumchan resize vectors, try to load
	numchan=realnumchan;
	virtualnumchan = 0;
	resizeVectors();
	tryToLoadConfigFile();
	
	//compute any possible virtual channels
	computevirtualchannels();

	cout << " done. Next refresh in 60 sec.\n";
	logoutput(" done. Next refresh in 60 sec.\n");

}//end loadaeconfigcsv







//this function loads the damagesum-numchan-scanrate.csv file and
//updates the max damage dial and damage table
void loaddamagesumcsv(void){
	float maxdamage=0;

	//build filename
	string filename;
	char temp[256];
	filename = "damagesum-";
	sprintf(temp,"%i",numchan);
	filename.append(temp);
	filename.append("-");
	sprintf(temp,"%i",(int)scanrate);
	filename.append(temp);
	filename.append(".csv");

	infile.clear();
	cout << "loading " << filename << " file:";
	logtimestamp();
	logoutput("loading ");
	logoutput(filename);
	logoutput(" file:");

	infile.open(filename.data());
	if(!infile.good()){
		cout << filename << " not found.\n";
		logoutput(" not found.\n");
		infile.close();
		return;
	}//end if
	infile >> numchan;

	//update damage channel browser
	string entry;
	char integer[256];
	infile.getline(temp,256,'\n');
	int colw[2];
	colw[0] = 200;
	colw[1] = 0;
	//channelbrowser->column_widths(colw);
	int position = channelbrowser->position();
	int position2 = damagescrollbar->value();

	//fill damage browser
	channelbrowser->clear();
	channelbrowser->add("ch\tname\tdamage",0);
	for(int i=0; i < numchan; i++){
		entry.clear();
		sprintf(integer,"%i",i);

		infile.getline(temp,256,'\n');

		if(!performrainflowanalysis[i]){
			entry.append("@-");
		}//end if
		else{
			if(atof(temp) >= 0.7 && atof(temp) < 1){
				entry.append("@B3@.");
			}//endif
			else if(atof(temp) >= 1){
				entry.append("@B128@.");
			}//end if
			if(atof(temp) > maxdamage){
				maxdamage = (float)atof(temp);
				maxdamagechanneloutput->value(channelnames[i].data());
			}//end if
		}//end else

		entry.append(integer);
		entry.append("\t");
		entry.append(channelnames[i]);
		entry.append("\t");
		entry.append(temp);

		channelbrowser->add(entry.data(),0);
	}//end for

	cout << " done. Next refresh in 60 sec.\n";
	logoutput(" done. Next refresh in 60 sec.\n");
	infile.close();

	channelbrowser->position(position);

	//dial and progress bar indicators
	damagedial->angles(36,324);
	damagedial->type(FL_LINE_DIAL);
	if(maxdamage > 1) damagedial->value(1);
	else damagedial->value(maxdamage);
	if(maxdamage < 0.7){
		damagedial->color2(2);
	}//end if
	else if(maxdamage >= 0.7 && maxdamage < 1){
		damagedial->color2(3);
	}//end if
	else if(maxdamage >= 1){
		damagedial->color2(128);
	}//end if
	//make sure gauge numbers are drawn on top of the dial after refresh
	db0->redraw();
	db1->redraw();
	db2->redraw();
	db3->redraw();
	db4->redraw();

	damagescrollbar->linesize(20);
	damagescrollbar->value(position2,1,0,numchan*20);

}//end loaddamagesumcsv




//this function loads statslog.csv and performs a whole bunch of stuff.
//it updates the damage rate dial, channel health table, channel checkboxes,
//and keeps track of scrollbar position.
void loadstatslogcsv(void){
	float maxdamagerate=0;
	char temp[256];

	infile.clear();
	cout << "loading statslog.csv file:";
	logtimestamp();
	logoutput("loading statslog.csv file:");

	infile.open("statslog.csv");
	if(!infile.good()){
		cout << "statslog.csv not found.\n";
		logoutput("statslog.csv not found.\n");
		infile.close();
		return;
	}//end if
	infile.close();

	scanrates.clear();
	numchannels.clear();

	//read the statslog
	statslog.clear();
	readstatslog("statslog.csv");
	cout << " done. Next refresh in 60 sec.\n";
	logoutput(" done. Next refresh in 60 sec.\n");

	determinescanratesandchannels();

	int position = channelhealth100hzbrowser->position();
	int position2 = channelcheckbrowser->position();
	int position3 = healthscrollbar->value();
	int position4 = channelscrollbar->value();

	//determine starting line for given scanrate to read through channel names
	unsigned int line=0;
	for(line = statslog.size()-1; line > 0 && statslog[line].scanrate != scanrate; line--){
	}//end for
	if(line != 0)line -= statslog[line].numchan-1;
	else if(line == 0 && (statslog[0].numchan != numchan || statslog[0].scanrate != scanrate)){
		cout << "ERROR: numchan-scanrate: " << numchan << "-" << scanrate << " not found in statslog.csv\n";
		logtimestamp();
		logoutput("ERROR: numchan-scanrate: ");
		sprintf(temp,"%i-%i not found in statslog.csv\n",numchan,(int)scanrate);
		logoutput(temp);
		return;
	}//end else if
	string entry;

	//determine checked boxes to recheck later
	vector<int> tempchecks;
	for(int i=1; i <= channelcheckbrowser->nitems(); i++){
		tempchecks.insert(tempchecks.end(),channelcheckbrowser->checked(i));
	}//end for

	//populate health browser
	channelcheckbrowser->clear();
	channelhealth100hzbrowser->clear();
	channelhealth100hzbrowser->add("ch\tname\tvariance\tmaximum\tminimum\tdamagerate",0);
	for(unsigned int i=line; i < line+statslog[line].numchan; i++){
		entry.clear();

		if(!performmmmanalysis[statslog[i].channel]){
			entry.append("@-");
		}//end if
		else if(statslog[i].variance < 0.01 && statslog[i].variance > 0){
			entry.append("@B3@.");
		}//endif
		else if(statslog[i].variance == 0 || statslog[i].max == 0 || statslog[i].min == 0){
			entry.append("@B128@.");
		}//end if
		else if(performrainflowanalysis[statslog[i].channel] && statslog[i].damage/(statslog[i].seconds/3600) >= 0.005 && statslog[i].damage/(statslog[i].seconds/3600) < 0.01){
			entry.append("@B3@.");
		}//end if
		else if(performrainflowanalysis[statslog[i].channel] && statslog[i].damage/(statslog[i].seconds/3600) >= 0.01){
			entry.append("@B128@.");
		}//end if

		sprintf(temp,"%i",statslog[i].channel);
		entry.append(temp);
		entry.append("\t");
		entry.append(statslog[i].channelname);
		entry.append("\t");
		sprintf(temp,"%f",statslog[i].variance);
		entry.append(temp);
		entry.append("\t");
		sprintf(temp,"%f",statslog[i].max);
		entry.append(temp);
		entry.append("\t");
		sprintf(temp,"%f",statslog[i].min);
		entry.append(temp);
		entry.append("\t");
		sprintf(temp,"%f",statslog[i].damage/(statslog[i].seconds/3600));
		entry.append(temp);
		channelhealth100hzbrowser->add(entry.data(),0);
		channelcheckbrowser->add(statslog[i].channelname.data(),0);

		if(performrainflowanalysis[statslog[i].channel] && maxdamagerate < statslog[i].damage/(statslog[i].seconds/3600)){
			maxdamagerate = statslog[i].damage/(statslog[i].seconds/3600);
			maxdamageratechanneloutput->value(channelnames[statslog[i].channel].data());
		}//end if
	}//end for

	//set damage rate dial
	damageratedial->angles(36,324);
	damageratedial->type(FL_LINE_DIAL);
	if(maxdamagerate > 0.01) damageratedial->value(0.01);
	else damageratedial->value(maxdamagerate);
	if(maxdamagerate < 0.005) damageratedial->color2(2);
	else if(maxdamagerate >= 0.005 && maxdamagerate < 0.01) damageratedial->color2(3);
	else damageratedial->color2(128);
	//damageratedial->redraw();
	drb0->redraw();
	drb1->redraw();
	drb2->redraw();
	drb3->redraw();
	drb4->redraw();
	
	for(int i=1; i <= channelcheckbrowser->nitems() && i-1 < (int)tempchecks.size(); i++){
		channelcheckbrowser->checked(i,tempchecks[i-1]);
	}//end for

	channelhealth100hzbrowser->position(position);
	channelcheckbrowser->position(position2);

	lastdatafileacquired->value(statslog[line].filename.data());
	lastdatafileanalysisdate->value(statslog[line].analysisdate.data());

	healthscrollbar->linesize(20);
	healthscrollbar->value(position3,1,0,numchan*20);

	channelscrollbar->linesize(20);
	channelscrollbar->value(position4,1,0,numchan*30);

}//end loadstatslogcsv






//the calls the different load functions in a specified order
void loadinputfiles(void*){
	loadaeconfigcsv();//loads config and determines virtual channels
	loaddamagesumcsv();//loads damage.csv; cumulative damage
	loadstatslogcsv();//loads statslog.csv; almost all stats

	Fl::repeat_timeout(60.0, loadinputfiles);//repeat every minute
}//end loadinputfiles






//this function plots trend data from statslog.csv with gnuplot
void plottrend(string type){

	float dataval;
	ofstream outfile;

	//write gnuplot.bat that will start gnuplot and read the macro
	outfile.open("gnuplot.bat", ios::trunc);
	outfile << "wgnuplot gnuplot.macro -\n";
	outfile.close();
	outfile.clear();

	//write gnuplot.macro file that tells gnuplot what to do
	outfile.open("gnuplot.macro", ios::trunc);
	outfile << "set title \"trend of " << type << " across all analyzed data files\"\n";
	outfile << "show title\n";
	outfile << "set xlabel \"analysis date and time (GMT)\"\n";
	outfile << "show xlabel\n";
	outfile << "set ylabel \"" << type << " value\"\n";
	outfile << "show ylabel\n";
	outfile << "set key outside\n";
	outfile << "set xdata time\n";
	outfile << "set format x \"%Y/%m/%d\\n%H:%M:%S\"\n";
	outfile << "set timefmt \"%s\"\n";	
	outfile << "plot ";
	for(int i=1,numchecks=0; i <= channelcheckbrowser->nitems(); i++){
		if(channelcheckbrowser->checked(i)){
			numchecks++;
			outfile << "\"gnuplotdata.dat\" using 1:" << numchecks+1 << " with linespoints title \"" << channelcheckbrowser->text(i) << "\"";
			if(numchecks < channelcheckbrowser->nchecked()) outfile << ",";
		}//end if
	}//end for
	outfile.close();
	outfile.clear();


	//write gnuplot.dat that contains the data to be plotted
	outfile.open("gnuplotdata.dat", ios::trunc);
	for(unsigned int i=0; i < statslog.size(); i++){
		if(statslog[i].scanrate == scanrate){
			if(statslog[i].channel == 0) outfile << statslog[i].analysistime << "\t";
			if(channelcheckbrowser->checked(statslog[i].channel+1)){
				if(type == "mean") dataval = statslog[i].mean;
				else if(type == "max") dataval = statslog[i].max;
				else if(type == "min") dataval = statslog[i].min;
				else if(type == "range") dataval = statslog[i].max-statslog[i].min;
				else if(type == "variance") dataval = statslog[i].variance;
				else if(type == "skewness") dataval = statslog[i].skewness;
				else if(type == "kurtosis") dataval = statslog[i].kurtosis;
				else if(type == "psdpeakfrequency") dataval = statslog[i].psdpeakfrequency;
				else if(type == "psd99pctfrequency") dataval = statslog[i].psd99pctfrequency;
				else if(type == "damage") dataval = statslog[i].damage;
				else if(type == "zvf") dataval = (float)statslog[i].zerovarianceflag;
				else if(type == "esf") dataval = (float)statslog[i].emptyscaleflag;
				else if(type == "ylf") dataval = (float)statslog[i].yellowlineflag;
				else if(type == "rlf") dataval = (float)statslog[i].redlineflag;
				else if(type == "fsf") dataval = (float)statslog[i].fullscaleflag;
				else if(type == "hlvf") dataval = (float)statslog[i].hplpvarianceflag;
				else if(type == "hlmf") dataval = (float)statslog[i].hplpmaxflag;
				else if(type == "mean-psz") dataval = statslog[i].mean-statslog[i].piersidezerovalue;
				outfile << dataval;
				if(channelcheckbrowser->nchecked() > 1) outfile << "\t";
			}//end if
			if(statslog[i].channel == numchan-1)
				outfile << endl;
		}//end if
	}//end for
	outfile.close();

	//run the command
	logtimestamp();
	logoutput("executing gnuplot.bat: ");
	system("start /b cmd /C gnuplot.bat");
	logoutput(" done\n");
}//end plottrend














//this function plots trend data from statslog.csv with gnuplot into a PDF file
//gnuplot plots to a postscript output file that is compressed into a PDF with ghostscript
void plotPDFtrend(string type,int channel){

	float dataval;
	ofstream outfile;

	//write gnuplot.bat file that runs gnuplot on a macro file
	outfile.open("gnuplot.bat", ios::trunc);
	outfile << "wgnuplot gnuplot.macro\n";
	outfile.close();
	outfile.clear();

	//write gnuplot.macro file that tells gnuplot what to do
	outfile.open("gnuplot.macro", ios::trunc);
	outfile << "set terminal postscript color\n";
	outfile << "set output \"gnuplot.ps\"\n";
	outfile << "set title \"trend of " << type << " across all analyzed " << scanrate << "Hz data files\"\n";
	outfile << "show title\n";

	if(type == "psdsurface"){
		outfile << "set xlabel \"Hz\"\n";
		outfile << "show xlabel\n";
		outfile << "set ylabel \"analysis date and time (GMT)\"\n";
		outfile << "show ylabel\n";
		outfile << "set zlabel \"" << type << " value\"\n";
		outfile << "show zlabel\n";
		outfile << "#set key outside\n";
		outfile << "#set ydata time\n";
		outfile << "#set format y \"%Y/%m/%d\\n%H:%M:%S\"\n";
		outfile << "#set timefmt \"%s\"\n";
		outfile << "set ytic " << (statslog[statslog.size()-1].analysistime-statslog[0].analysistime)/5 << endl;
		outfile << "set pm3d\n";
	}//end if
	else{
		outfile << "set xlabel \"analysis date and time (GMT)\"\n";
		outfile << "show xlabel\n";
		outfile << "set ylabel \"" << type << " value\"\n";
		outfile << "show ylabel\n";
		outfile << "set key outside\n";
		outfile << "set xdata time\n";
		outfile << "set format x \"%Y/%m/%d\\n%H:%M:%S\"\n";
		outfile << "set timefmt \"%s\"\n";
		outfile << "set xtic " << (statslog[statslog.size()-1].analysistime-statslog[0].analysistime)/5 << endl;
	}//end else

	if(type == "mmm"){
		outfile << "plot \"gnuplotdata.dat\" using 1:2 with linespoints title \"" << channelnames[channel] << "-mean\"";
		outfile << ", \"gnuplotdata.dat\" using 1:3 with linespoints title \"" << channelnames[channel] << "-max\"";
		outfile << ", \"gnuplotdata.dat\" using 1:4 with linespoints title \"" << channelnames[channel] << "-min\"";
		outfile << ", " << piersidezerovalues[channel] << " title \"" << channelnames[channel] << "-PSZ\"";
		outfile << ", " << yellowlinevalues[channel] << " title \"" << channelnames[channel] << "-YLV\"";
		outfile << ", " << redlinevalues[channel] << " title \"" << channelnames[channel] << "-RLV\"";
	}//end if
	else if(type == "flags"){
		outfile << "plot \"gnuplotdata.dat\" using 1:2 with linespoints title \"" << channelnames[channel] << "-zvf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:3 with linespoints title \"" << channelnames[channel] << "-esf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:4 with linespoints title \"" << channelnames[channel] << "-ylf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:5 with linespoints title \"" << channelnames[channel] << "-rlf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:6 with linespoints title \"" << channelnames[channel] << "-fsf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:7 with linespoints title \"" << channelnames[channel] << "-hlvf\"";
		outfile << ", \"gnuplotdata.dat\" using 1:8 with linespoints title \"" << channelnames[channel] << "-hlmf\"";
	}//end if
	else if(type == "cumulativedamage"){
		outfile << "plot \"gnuplotdata.dat\" using 1:2 with linespoints title \"" << channelnames[channel] << "-sumdam\"";
		outfile << ", " << 0.7 << " title \"0.7\"";
		outfile << ", " << 1 << " title \"1.0\"";
		outfile << ", " << 2 << " title \"2.0\"";
	}//end if
	else if(type == "psd"){
		outfile << "plot \"gnuplotdata.dat\" using 1:2 with linespoints title \"" << channelnames[channel] << "-psdpeakfreq\"";
		outfile << ", \"gnuplotdata.dat\" using 1:3 with linespoints title \"" << channelnames[channel] << "-psd99pctfreq\"";
	}//end if
	else if(type == "psdsurface"){
		outfile << "splot \"gnuplotdata.dat\" with pm3d title \"" << channelnames[channel] << "-psd\"";
	}//end if
	else{
		outfile << "plot \"gnuplotdata.dat\" using 1:2 with linespoints title \"" << channelnames[channel] << "\"";
	}//end else

	outfile.close();
	outfile.clear();


	//write gnuplot.dat that contains the data to be plotted
	outfile.open("gnuplotdata.dat", ios::trunc);
	double cumulativedamage=0;
	for(unsigned int i=0; i < statslog.size(); i++){
		if(statslog[i].scanrate == scanrate){
			if(channel == statslog[i].channel){
				if(type != "psdsurface")outfile << statslog[i].analysistime << "\t";

				if(type == "mean") dataval = statslog[i].mean;
				else if(type == "max") dataval = statslog[i].max;
				else if(type == "min") dataval = statslog[i].min;
				else if(type == "range") dataval = statslog[i].max-statslog[i].min;
				else if(type == "variance") dataval = statslog[i].variance;
				else if(type == "skewness") dataval = statslog[i].skewness;
				else if(type == "kurtosis") dataval = statslog[i].kurtosis;
				else if(type == "psdpeakfrequency") dataval = statslog[i].psdpeakfrequency;
				else if(type == "damage") dataval = statslog[i].damage;
				else if(type == "zvf") dataval = (float)statslog[i].zerovarianceflag;
				else if(type == "esf") dataval = (float)statslog[i].emptyscaleflag;
				else if(type == "ylf") dataval = (float)statslog[i].yellowlineflag;
				else if(type == "rlf") dataval = (float)statslog[i].redlineflag;
				else if(type == "fsf") dataval = (float)statslog[i].fullscaleflag;
				else if(type == "hlvf") dataval = (float)statslog[i].hplpvarianceflag;
				else if(type == "hlmf") dataval = (float)statslog[i].hplpmaxflag;
				else if(type == "mean-psz") dataval = statslog[i].mean-statslog[i].piersidezerovalue;
				
				if(type == "mmm"){
					outfile << statslog[i].mean << "\t" << statslog[i].max << "\t" << statslog[i].min << endl;
				}//end if
				else if(type == "flags"){
					outfile << statslog[i].zerovarianceflag << "\t" << statslog[i].emptyscaleflag << "\t" << statslog[i].yellowlineflag << "\t" << statslog[i].redlineflag << "\t" << statslog[i].fullscaleflag << "\t" << statslog[i].hplpvarianceflag << "\t" << statslog[i].hplpmaxflag << endl;
				}//end if
				else if(type == "cumulativedamage"){
					cumulativedamage += statslog[i].damage;
					outfile << cumulativedamage << endl;
				}//end if
				if(type == "psd"){
					outfile << statslog[i].psdpeakfrequency << "\t" << statslog[i].psd99pctfrequency << endl;
				}//end if
				if(type == "psdsurface"){
					int originalnumchan = numchan;
					int originalrealnumchan = realnumchan;
					float originalscanrate = scanrate;

					//load psd file
					data.clear();
					cals.clear();
					string tempfilename = statslog[i].filename;
					tempfilename[tempfilename.size()-4] = '-';
					tempfilename[tempfilename.size()-3] = 'p';
					tempfilename[tempfilename.size()-2] = 's';
					tempfilename[tempfilename.size()-1] = 'd';
					tempfilename.append(".scl");
					cout << "loading: " << tempfilename << " channel: " << channel << endl;
					loadSCLfile(tempfilename);

					//output psd file into proper splot 3d format
					for(unsigned int f=0; f < data[channel].size(); f++){
						outfile << f/scanrate << "\t" << statslog[i].analysistime << "\t" << data[channel][f] << endl;
					}//end for

					//output line between 3d blocks
					outfile << endl;

					numchan = originalnumchan;
					realnumchan = originalrealnumchan;
					scanrate = originalscanrate;
				}//end if
				else{
					outfile << dataval << endl;
				}//end else
			}//end if
		}//end if
	}//end for
	outfile.close();

	//run gnuplot command to generate postscript plot
	logtimestamp();
	logoutput("executing gnuplot.bat: ");
	system("start /b cmd /C gnuplot.bat");
	logoutput(" done\n");

	//generate new command to convert postscript output to pdf
	char scanratetext[256];
	char channeltext[256];
	string command = "ps2pdf gnuplot.ps gnuplot-";
	sprintf(scanratetext,"%i",(int)scanrate);
	command.append(scanratetext);
	command.append("-");
	sprintf(channeltext,"%i",channel);
	string leadingzeros = "00";
	string leadingzero = "0";
	if(channel < 10) command.append(leadingzeros);
	else if(channel < 100) command.append(leadingzero);
	command.append(channeltext);
	command.append("-");
	if(type == "mmm")
		command.append("0");
	else if(type == "psd")
		command.append("1");
	else if(type == "psdsurface")
		command.append("2");
	else if(type == "flags")
		command.append("3");
	else if(type == "cumulativedamage")
		command.append("4");
	command.append(type);
	command.append(".pdf");
	
	//run ps2pdf command
	logtimestamp();
	logoutput(command);
	system(command.data());
	logoutput(" done\n");

	cout << "scanrate: " << scanrate << " channel: " << channel << " type: " << type << endl;
}//end plotPDFtrend








//generatePDFreport
//automatically generates a pdf report containing hundreds of plots
//that indicate the health of various measurements
void generatePDFreport(void*){
	//delete existing system report
	system("del SystemReport.pdf");

	//select slow system radio button
	slowbutton();
	//gen plots
	for(int c=0; c < numchan; c++)
		plotPDFtrend("mmm",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("psd",c);
	//for(int c=0; c < numchan; c++)
		//plotPDFtrend("psdsurface",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("flags",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("cumulativedamage",c);

	//select system
	medbutton();
	//gen plots
	for(int c=0; c < numchan; c++)
		plotPDFtrend("mmm",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("psd",c);
	//for(int c=0; c < numchan; c++)
		//plotPDFtrend("psdsurface",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("flags",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("cumulativedamage",c);

	//select system
	fastbutton();
	//gen plots
	for(int c=0; c < numchan; c++)
		plotPDFtrend("mmm",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("psd",c);
	//for(int c=0; c < numchan; c++)
		//plotPDFtrend("psdsurface",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("flags",c);
	for(int c=0; c < numchan; c++)
		plotPDFtrend("cumulativedamage",c);

	//select slow system radio button to end with
	slowradiobutton->setonly();
	slowbutton();

	//conatenate pdfs together using pdftk
	logtimestamp();
	logoutput("pdftk gnuplot-*.pdf cat output SystemReport.pdf");
	system("pdftk gnuplot-*.pdf cat output SystemReport.pdf");
	//system("del gnuplot*.pdf gnuplot.ps");
	logoutput(" done\n");

	//unsigned int timeoutday = 3600*24;
	//Fl::repeat_timeout(timeoutday, generatePDFreport);
}//end generatePDFreport







//this function plots selected channels (time histories) from the last data file with gnuplot.
//this funciton loads the scl file, then outputs an ascii tab-delimited file
//containting only the channels of interest. the data in memory will only be
//reloaded from an scl file if the last acquired data file has changed.
void plotchannels(string filename, string xlabel){
	//try to load the proper data file
	float tempscanrate = scanrate;
	cout << "loading scl file for plot: ";
	logtimestamp();
	logoutput("loading scl file for plot: ");
	if(loadeddatafile != filename){
		data.clear();
		cals.clear();
		int temprealnumchan = realnumchan;
		loadSCLfile(filename);
		realnumchan = temprealnumchan;
		sclscanrate = scanrate;
		scanrate = tempscanrate;
		loadeddatafile = filename;
		if(xlabel != "Hz")applysfo();
	}//end if


	cout << " done.\n";
	logoutput(" done.\n");

	ofstream outfile;

	//write gnuplot command
	outfile.open("gnuplot.bat", ios::trunc);
	outfile << "wgnuplot gnuplot.macro -\n";
	outfile.close();
	outfile.clear();

	//write gnuplot macro that tells gnuplot what to do
	outfile.open("gnuplot.macro", ios::trunc);
	string tempstring = filename;
	for(unsigned int i=0; i < tempstring.size(); i++)
		if(tempstring[i] == '\\') tempstring[i] = '/';
	if(!quickplot->value())
		outfile << "set title \"" << tempstring << "\"\n";
	else
		outfile << "set title \"quick plot " << tempstring << "\"\n";
	outfile << "show title\n";
	outfile << "set xlabel \"" << xlabel << "\"\n";
	outfile << "show xlabel\n";
	outfile << "set ylabel \"value\"\n";
	outfile << "show ylabel\n";
	outfile << "set key outside\n";
	outfile << "plot ";
	for(int i=1,numchecks=0; i <= channelcheckbrowser->nitems(); i++){
		if(channelcheckbrowser->checked(i) && i <= realnumchan){
			numchecks++;
			outfile << "\"gnuplotdata.dat\" using 1:" << numchecks+1 << " with lines title \"" << channelcheckbrowser->text(i) << "\"";
			if(numchecks < channelcheckbrowser->nchecked()) outfile << ",";
		}//end if
	}//end for
	outfile.close();
	outfile.clear();

	//determine channels to output
	vector<int> dispchannels;
	for(int c=1; c <= channelcheckbrowser->nitems(); c++){
		if(channelcheckbrowser->checked(c) && c <= realnumchan){
			dispchannels.insert(dispchannels.end(),c-1);
		}//end if
	}//end for

	cout << "saving scl data to ascii for plot: ";
	logtimestamp();
	logoutput("saving scl data to ascii for plot: ");
	outfile.open("gnuplotdata.dat", ios::trunc);

	//output data to be plotted
	unsigned int step=1;
	if(quickplot->value()) step = (unsigned int)sclscanrate;
	for(unsigned int i=0; i < data[0].size(); i+=step){
		outfile << (float)i/sclscanrate << "\t";
		for(unsigned int c=0; c < dispchannels.size(); c++){
			outfile << data[dispchannels[c]][i] << "\t";
		}//end for
		outfile << endl;
	}//end for

	outfile.close();
	outfile.clear();
	cout << " done.\n";
	logoutput(" done.\n");

	//execute gnuplot
	logtimestamp();
	logoutput("executing gnuplot.bat: ");
	system("start /b cmd /C gnuplot.bat");
	logoutput(" done\n");
	scanrate = tempscanrate;
}//end plotchannels























//this function opens a pdf file based on a standard filename format.
//this allows drawings to be added on the fly without recompiling.
//the drawing filename spec is: drawing-numchan-scanrate-chan.pdf.
//drawings are opened with Adobe Reader, which must be in the system path.
void displaygagelocationdrawing(void){
	string command;
	string filename;
	char nc[256];
	char sr[256];
	char temp[256];
	sprintf(nc,"%i",realnumchan);
	sprintf(sr,"%i",(int)scanrate);

	//try to open the drawing for each selected channel
	for(int i=1; i < channelcheckbrowser->nitems(); i++){
		command.clear();
		filename.clear();
		if(channelcheckbrowser->checked(i)){
			cout << "loading drawing for " << channelcheckbrowser->text(i) << ": ";
			logtimestamp();
			logoutput("loading drawing for ");
			logoutput(channelcheckbrowser->text(i));
			logoutput(": ");

			//build the command and drawing file name
			command = "start /b acrord32 ";
			filename = "drawing-";
			filename.append(nc);
			filename.append("-");
			filename.append(sr);
			filename.append("-");
			sprintf(temp,"%i",i-1);
			filename.append(temp);
			filename.append(".pdf");

			cout << filename << " :";
			logoutput(filename);
			logoutput(" :");

			//execute the command that opens the drawing in acrobat
			command.append(filename);
			system(command.data());

			cout << " done.\n";
			logoutput(" done.\n");
		}//end if
	}//end for
}//end dsiaplaygagelocationdrawing







//opens the last scl file in IGOR
//nswccd-igor macros required
//this loads igor and passes some command line arguments that tell igor to execute
//a few commands once it is open
void calligor(int type){
	//build data file name
	string datafilename = lastdatafileacquired->value();
	if(type == 1){
		datafilename[datafilename.size()-4] = '-';
		datafilename[datafilename.size()-3] = 'p';
		datafilename[datafilename.size()-2] = 's';
		datafilename[datafilename.size()-1] = 'd';
		datafilename.append(".scl");
	}//end if
	else if(type == 2){
		datafilename[datafilename.size()-4] = '-';
		datafilename[datafilename.size()-3] = 'r';
		datafilename[datafilename.size()-2] = 'a';
		datafilename[datafilename.size()-1] = 'o';
		datafilename.append(".scl");
	}//end else

	//copy the data file to c:\igor.scl
	//this must be done because many of the file references here are filenames and not full path names
	//igor requires full path names because its working directory is c:\Program Files\WaveMetrics\Igor Pro Folder
	string command = "copy \"";
	command.append(datafilename);
	command.append("\" c:\\igor.scl");
	cout << command << endl;
	system(command.data());

	//execute igor and then loadGenericSCLFile function from nswccd65igor macros on c:\igor.scl data file
	command = "start /b Igor.exe /X \"loadGenericSCLFileSP4BHBF(\"c:\\igor.scl\")\"";
	cout << command << endl;
	system(command.data());
}//end calligor







//this function does everything needed when switching to slow system data
//it resets numchan and scanrate to the proper values and then
//calls functions to reload all necessary input files
void slowbutton(void){
	cout << "slow button pressed\n";
	logtimestamp();
	logoutput("slow button pressed\n");

	realnumchan = 79;
	virtualnumchan = 0;
	numchan = realnumchan + virtualnumchan;
	scanrate = 100;

	Fl::remove_timeout(loadinputfiles);
	loadinputfiles(0);
}//end slowbutton



//this function does everything needed when switching to medium system data
//it resets numchan and scanrate to the proper values and then
//calls functions to reload all necessary input files
void medbutton(void){
	cout << "med button pressed\n";
	logtimestamp();
	logoutput("med button pressed\n");

	realnumchan = 50;
	virtualnumchan = 0;
	numchan = realnumchan + virtualnumchan;
	scanrate = 1000;
	
	Fl::remove_timeout(loadinputfiles);
	loadinputfiles(0);
}//end medbutton



//this function does everything needed when switching to fast system data
//it resets numchan and scanrate to the proper values and then
//calls functions to reload all necessary input files
void fastbutton(void){
	cout << "fast button pressed\n";
	logtimestamp();
	logoutput("fast button pressed\n");

	realnumchan = 20;
	virtualnumchan = 0;
	numchan = realnumchan + virtualnumchan;
	scanrate = 5000;
	
	Fl::remove_timeout(loadinputfiles);
	loadinputfiles(0);
}//end fastbutton









//this function logs the date and time stamp followed by a tab
//and makes sure the scrollbar is at the end of the log
void logtimestamp(void){
	logtext.clear();
	currenttime = time(0);
	logtext = ctime(&currenttime);
	logtext[logtext.size()-1] = '\t';
	logbuffer->append(logtext.data());
	logdisplay->scroll(logdisplay->count_lines(0,logbuffer->length(),1),0);
}//end logfiletimestamp




//this function logs the input string
//and makes sure the scrollbar is at the end of the log
void logoutput(string output){
	logtext.clear();
	logtext = output;
	logbuffer->append(logtext.data());
	logdisplay->scroll(logdisplay->count_lines(0,logbuffer->length(),1),0);
}//end logfileoutput















int main(int argc, char **argv) {

	//load window made with fluid gui designer
	//this function is automatically generated and
	//defined in guitest-fluid.h and guitest-fluid.cxx
	window = make_window();

	//finish setting up gui and show it
	window->end();
	window->show(argc, argv);

	//set up log buffer and log text display
	//with finger size touchscreen controls
	logbuffer = new Fl_Text_Buffer(0);
	logdisplay->buffer(logbuffer);
	logdisplay->scrollbar_width(75);

	//initial log output
	cout << "guitest started\n";
	logtimestamp();
	logoutput("guitest started\n");

	//select slow system radio button initially
	slowradiobutton->setonly();
	slowbutton();

	//start program
	return Fl::run();
	
}//end main

