/* scl2ascii.cpp - lastupdate 2006-07-11-16-14
Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com

Converts scl binary data files to ascii comma delimited data files
*/

#include"../../ae/ae/ae.h"

int filenumber = 0;//current file number
int originalnumchan = 0;//number of channels for the first file


//This function outputs the data for the input data file
//as a ascii comma delimited file.
//The output psd file name is the same as the input file name
//with the extension changed to .csv.
//Example: input: run_77.scl outupt: run_77.csv
//The resulting csv file can be loaded directly into IGOR or Excel.
void outputCSVfile(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -psd added
	string fname = filename;
	fname[fname.size()-3] = 'c';
	fname[fname.size()-2] = 's';
	fname[fname.size()-1] = 'v';

	//open and overwrite any existing output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc);

	//ascii header
	outputfile << "scan rate," << scanrate << endl;
	//scl header cal factors identical to input data file
	for(i=0; i < (unsigned int)numchan; i++){
		outputfile << cals[i];
		if(i != numchan-1){
			outputfile << ",";
		}//end if
	}//end for
	outputfile << endl;
	//channel names
	for(i=0; i < (unsigned int)numchan; i++){
		outputfile << "ch" << i;
		if(i != numchan-1){
			outputfile << ",";
		}//end if
	}//end for
	outputfile << endl;

	//scl interleaved data
	for(i=0; i < data[0].size(); i++){
		for(j=0; j < (unsigned int)numchan; j++){
			outputfile << data[j][i];
			if(j != numchan-1){
				outputfile << ",";
			}//end if
		}//end for
		outputfile << endl;
	}//end for

	outputfile.close();

}//end outputSCLfile


//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "starttime:\t" << starttime << endl;

	for(filenumber=0; filenumber < argc-1; filenumber++){
		//load input data file
		cout << "filename:\t" << argv[filenumber+1] << endl;
		cout << "reading time histories: ";
		loadSCLfile(argv[filenumber+1]);
		if(filenumber ==0) originalnumchan = numchan;
		else if( numchan != originalnumchan) cout << "Error: numchan does not match originalnumchan\n";
		cout << "done\n";
		cout << "number of channels:\t" << numchan << endl;
		cout << "scan rate:\t" << scanrate << endl;
		cout << "total scans:\t" << data[0].size() << endl;
		cout << "seconds:\t" << data[0].size()/scanrate << endl;

		//write output file
		cout << "writing output file: ";
		outputCSVfile(argv[filenumber+1]);
		cout << "done\n";

		//clean up
		for(int i=0;i < numchan; i++){
			data[i].resize(0);//clear arrays if this is not the first file
		}//end for
		data.resize(0);
		cals.resize(0);
	}//end for

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;

	return 0;
}//end main


