/* sclconcat.cpp - lastupdate 2006-07-13-21-14
Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com

Concatenates scl files together
*/

#include"../../ae/ae/ae.h"


//global variables
int filenumber = 0;//current file number
int originalnumchan = 0;//number of channels for the first file
unsigned int totalsize = 0;


//This function outputs the data for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output psd file name is the same as the input file name
//with the extension changed to .scl.
//Example: input: run_77.dat outupt: run_77.scl
//The resulting scl file can be loaded directly into IGOR.
void outputSCLfile(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -psd added
	string fname = filename;
	fname[fname.size()-4] = '-';
	fname[fname.size()-3] = 'c';
	fname[fname.size()-2] = 'o';
	fname[fname.size()-1] = 'n';
	fname.append("cat.scl");

	//open and overwrite any existing binary output file
	ofstream outputfile;
	if(filenumber == 0){
		outputfile.open(fname.data(), ios::trunc|ios::binary);
		
		//scl header number of channels
		int tempnumchan = swapbytes(numchan);
		outputfile.write((char*)&tempnumchan,4);

		//scl header scan rate
		float tempscanrate = swapbytes(scanrate);
		outputfile.write((char*)&tempscanrate,4);

		//scl header cal factors identical to input data file
		double tempcal;
		for(i=0; i < (unsigned int)numchan; i++){
			tempcal = swapbytes(cals[i]);
			outputfile.write((char*)&tempcal,8);
		}//end for
	}//end if
	else
		outputfile.open(fname.data(), ios::app|ios::binary);

	//scl interleaved data
	for(i=0; i < data[0].size(); i++){
		for(j=0; j < (unsigned int)numchan; j++){
			datapt = swapbytes(data[j][i]);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputSCLfile


//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "starttime:\t" << starttime << endl;

	for(filenumber=0; filenumber < argc-1; filenumber++){
		//load input data file
		cout << "filename:\t" << argv[filenumber+1] << endl;
		cout << "reading time histories: ";
		loadSCLfile(argv[filenumber+1]);
		if(filenumber ==0) originalnumchan = numchan;
		else if( numchan != originalnumchan) cout << "Error: numchan does not match originalnumchan\n";
		cout << "done\n";
		cout << "number of channels:\t" << numchan << endl;
		cout << "scan rate:\t" << scanrate << endl;
		cout << "total scans:\t" << data[0].size() << endl;
		cout << "seconds:\t" << data[0].size()/scanrate << endl;
		totalsize += data[0].size();

		//write output file
		cout << "writing output file: ";
		outputSCLfile(argv[1]);
		cout << "done\n";

		//clean up
		for(int i=0;i < numchan; i++){
			data[i].resize(0);//clear arrays if this is not the first file
		}//end for
		data.resize(0);
		cals.resize(0);
	}//end for

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;
	cout << "total scans:\t" << totalsize << endl;
	cout << "total seconds:\t" << totalsize/scanrate << endl;
	cout << "daq time / analysis time:\t" << totalsize/scanrate/(endtime-starttime) << endl;

	return 0;
}//end main


