/* sclscanrate.cpp - lastupdate 200612041250
Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com

Changes the scanrate of one or more scl input files
*/

#include"../../ae/ae/ae.h"


//global variables
float newscanrate = 5000;//default new scan rate
int filenumber = 0;//current file number
int originalnumchan = 0;//number of channels for the first file



//This function outputs the data for the input data file
//as a SCL binary file to save storage space and CPU time.
//The output psd file name is the same as the input file name
//with the extension changed to .scl.
//Example: input: run_77.dat outupt: run_77.scl
//The resulting scl file can be loaded directly into IGOR.
void outputSCLfile(string filename){

	unsigned int i = 0;
	unsigned int j = 0;

	//generate new file name with -psd added
	string fname = filename;
	fname[fname.size()-4] = '-';
	fname[fname.size()-3] = 's';
	fname[fname.size()-2] = 'c';
	fname[fname.size()-1] = 'a';
	fname.append("nrate.scl");

	//open and overwrite any existing binary output file
	ofstream outputfile;
	outputfile.open(fname.data(), ios::trunc|ios::binary);
	
	//scl header number of channels
	int tempnumchan = swapbytes(numchan);
	outputfile.write((char*)&tempnumchan,4);

	//scl header scan rate
	float tempscanrate = swapbytes(newscanrate);
	outputfile.write((char*)&tempscanrate,4);

	//scl header cal factors identical to input data file
	double tempcal;
	for(i=0; i < (unsigned int)numchan; i++){
		tempcal = swapbytes(cals[i]);
		outputfile.write((char*)&tempcal,8);
	}//end for

	//scl interleaved data
	for(i=0; i < data[0].size(); i++){
		for(j=0; j < (unsigned int)numchan; j++){
			datapt = swapbytes(data[j][i]);
			outputfile.write((char*)&datapt, 4);
		}//end for
	}//end for

	outputfile.close();

}//end outputSCLfile


//load newscanrate.csv file
void loadNewScanrate(void){
	ifstream infile;
	infile.open("newscanrate.csv");
	if(infile.good()){
		char text[256];
		infile.getline(text,256,'\n');
		newscanrate = (float)atof(text);
	}//end if
	infile.close();
}//end loadNewScanrate

//save newscanrate.csv file
void saveNewScanrate(void){
	ofstream outfile;
	outfile.open("newscanrate.csv",ios::trunc);
	outfile << newscanrate << endl;
	outfile.close();
}//end saveNewScanrate


//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "starttime:\t" << starttime << endl;

	//load and/or create newscanrate.csv file
	loadNewScanrate();
	saveNewScanrate();

	for(filenumber=0; filenumber < argc-1; filenumber++){
		//load input data file
		cout << "filename:\t" << argv[filenumber+1] << endl;
		cout << "reading time histories: ";
		loadSCLfile(argv[filenumber+1]);
		if(filenumber ==0) originalnumchan = numchan;
		else if( numchan != originalnumchan) cout << "Error: numchan does not match originalnumchan\n";
		cout << "done\n";
		cout << "number of channels:\t" << numchan << endl;
		cout << "scan rate:\t" << scanrate << endl;
		cout << "total scans:\t" << data[0].size() << endl;
		cout << "seconds:\t" << data[0].size()/scanrate << endl;

		//write output file
		cout << "writing output file: ";
		outputSCLfile(argv[filenumber+1]);
		cout << "done\n";

		//clean up
		for(int i=0;i < numchan; i++){
			data[i].resize(0);//clear arrays if this is not the first file
		}//end for
		data.resize(0);
		cals.resize(0);
	}//end for

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;

	return 0;
}//end main


