//#define slaeversion 200701101232//version of this code YYYYMMDDHHMM
/*
slae.cpp

Gregory Alan Hildstrom
Hildstrom Engineering
http://www.hildstrom.com
Subcontract/D.O. DDL 04-D-4032-010
DDL Omni Engineering
Prime Contract N00178-04-D-4032, D.O. FD02
NSWCCD Code 653

JHSV Analysis Engine - NSWCCD-65-TR-2006/15
Stats Log Analysis Engine

This program performs statistical analysis on the statslog.csv AE output file.

If you are attempting to modify this code, it is best to start with the main() function in order to uderstand
what the program does from a high-level standpoint first.
*/

/* TODO
*/


#include"slae.h"
#include"../../ae/ae/ae.h"


string slaeversion = "200701101232";//version of this code YYYYMMDDHHMM


//performs linear regression using the least squares method
void LinearRegressionLeastSquares(vector<float>& data, double& lryint, double& lrbeta, double& lrcorrelation, double& lrcharval){
	if(data.size() <= 1){
		lryint = 0;
		lrbeta = 0;
		lrcorrelation = 0;
		lrcharval = 0;
		return;
	}//end if

	double sumx = 0;
	double sumxsq = 0;
	double sumy = 0;
	double sumysq = 0;
	double sumxy = 0;

	vector<float> xdata;
	for(unsigned int i=0; i < data.size(); i++){
		xdata.insert(xdata.end(),i);
	}//end for

	for(unsigned int i=0; i < data.size(); i++){
		sumx += xdata[i];
		sumxsq += xdata[i]*xdata[i];
		sumy += data[i];
		sumysq += data[i] * data[i];
		sumxy += xdata[i] * data[i];
	}//end for

	lryint = (sumy*sumxsq - sumx*sumxy)/((double)data.size()*sumxsq - sumx*sumx);

	lrbeta = ((double)data.size()*sumxy - sumx*sumy)/((double)data.size()*sumxsq - sumx*sumx);

	lrcorrelation = ((double)data.size()*sumxy - sumx*sumy)/sqrt(((double)data.size()*sumxsq - sumx*sumx)*((double)data.size()*sumysq - sumy*sumy));
	lrcorrelation *= lrcorrelation;

	lrcharval = exp(-lryint/lrbeta);

}//end Linear Regression Least Squares


//computes statistics using the statistics in statslog.csv
void statsmmms(string filename){
	float scanrate;
	int numchan;

	double mean;
	float max;
	float min;
	double variance;
	double skew;
	double kurtosis;

	double lryint;
	double lrbeta;
	double lrcorrelation;
	double lrcharval;

	//open a new output file
	ofstream output;
	output.open("statsstatslog.csv",ios::trunc);

	output << "intputfile,";
	output << "numfiles,";
	output << "scanrate,";
	output << "channel,";
	output << "channelname,";

	output << "meanofmeans,";
	output << "maxofmeans,";
	output << "maxmeanfilename,";
	output << "minofmeans,";
	output << "minmeanfilename,";
	output << "varianceofmeans,";
	output << "skewofmeans,";
	output << "kurtosisofmeans,";
	output << "lryintofmeans,";
	output << "lrbetaofmeans,";
	output << "lrcorrelationofmeans,";

	output << "meanofmaxes,";
	output << "maxofmaxes,";
	output << "maxmaxfilename,";
	output << "minofmaxes,";
	output << "minmaxfilename,";
	output << "varianceofmaxes,";
	output << "skewofmaxes,";
	output << "kurtosisofmaxes,";
	output << "lryintofmaxes,";
	output << "lrbetaofmaxes,";
	output << "lrcorrelationofmaxes,";

	output << "meanofmins,";
	output << "maxofmins,";
	output << "maxminfilename,";
	output << "minofmins,";
	output << "minminfilename,";
	output << "varianceofmins,";
	output << "skewofmins,";
	output << "kurtosisofmins,";
	output << "lryintofmins,";
	output << "lrbetaofmins,";
	output << "lrcorrelationofmins,";

	output << "meanofvariances,";
	output << "maxofvariances,";
	output << "maxvariancefilename,";
	output << "minofvariances,";
	output << "minvariancefilename,";
	output << "varianceofvariances,";
	output << "skewofvariances,";
	output << "kurtosisofvariances,";
	output << "lryintofvariances,";
	output << "lrbetaofvariances,";
	output << "lrcorrelationofvariances,";

	output << "meanofskews,";
	output << "maxofskews,";
	output << "maxskewfilename,";
	output << "minofskews,";
	output << "minskewfilename,";
	output << "varianceofskews,";
	output << "skewofskews,";
	output << "kurtosisofskews,";
	output << "lryintofskews,";
	output << "lrbetaofskews,";
	output << "lrcorrelationofskews,";

	output << "meanofkurtosi,";
	output << "maxofkurtosi,";
	output << "maxkurtosisfilename,";
	output << "minofkurtosi,";
	output << "minkurtosisfilename,";
	output << "varianceofkurtosi,";
	output << "skewofkurtosi,";
	output << "kurtosisofkurtosi,";
	output << "lryintofkurtosi,";
	output << "lrbetaofkurtosi,";
	output << "lrcorrelationofkurtosi,";

	output << "meanofcals,";
	output << "maxofcals,";
	output << "maxcalfilename,";
	output << "minofcals,";
	output << "mincalfilename,";
	output << "varianceofcals,";
	output << "skewofcals,";
	output << "kurtosisofcals,";
	output << "lryintofcals,";
	output << "lrbetaofcals,";
	output << "lrcorrelationofcals,";

	output << "meanofpsdpeakfreqs,";
	output << "maxofpsdpeakfreqs,";
	output << "maxpsdpeakfreqfilename,";
	output << "minofpsdpeakfreqs,";
	output << "minpsdpeakfreqfilename,";
	output << "varianceofpsdpeakfreqs,";
	output << "skewofpsdpeakfreqs,";
	output << "kurtosisofpsdpeakfreqs,";
	output << "lryintofpsdpeakfreqs,";
	output << "lrbetaofpsdpeakfreqs,";
	output << "lrcorrelationofpsdpeakfreqs,";

	output << "meanofscalefactors,";
	output << "maxofscalefactors,";
	output << "maxscalefactorfilename,";
	output << "minofscalefactors,";
	output << "minscalefactorfilename,";
	output << "varianceofscalefactors,";
	output << "skewofscalefactors,";
	output << "kurtosisofscalefactors,";
	output << "lryintofscalefactors,";
	output << "lrbetaofscalefactors,";
	output << "lrcorrelationofscalefactors,";

	output << "meanofoffsets,";
	output << "maxofoffsets,";
	output << "maxoffsetfilename,";
	output << "minofoffsets,";
	output << "minoffsetfilename,";
	output << "varianceofoffsets,";
	output << "skewofoffsets,";
	output << "kurtosisofoffsets,";
	output << "lryintofoffsets,";
	output << "lrbetaofoffsets,";
	output << "lrcorrelationofoffsets,";

	output << "meanofdamage,";
	output << "maxofdamage,";
	output << "maxdamagefilename,";
	output << "minofdamage,";
	output << "mindamagefilename,";
	output << "varianceofdamage,";
	output << "skewofdamage,";
	output << "kurtosisofdamage,";
	output << "lryintofdamage,";
	output << "lrbetaofdamage,";
	output << "lrcorrelationofdamage,";

	output << "sumofflags,";
	output << "lastzerovarianceflagfile,";
	output << "lastemptyscaleflagfile,";
	output << "lastyellowlineflagfile,";
	output << "lastredlineflagfile,";
	output << "lastfullscaleflagfile,";
	output << "lasthplpvarianceflagfile,";
	output << "lasthplpmaxflagfile,";

	output << "meanofpsd99pctfreqs,";
	output << "maxofpsd99pctfreqs,";
	output << "maxpsd99pctfreqfilename,";
	output << "minofpsd99pctfreqs,";
	output << "minpsd99pctfreqfilename,";
	output << "varianceofpsd99pctfreqs,";
	output << "skewofpsd99pctfreqs,";
	output << "kurtosisofpsd99pctfreqs,";
	output << "lryintofpsd99pctfreqs,";
	output << "lrbetaofpsd99pctfreqs,";
	output << "lrcorrelationofpsd99pctfreqs\n";

	vector<float> tempdata;
	string datafilename;
	string tempchannelname;
	//compute stats on the stats and output to output file
	for(unsigned int i=0; i < scanrates.size(); i++){
		scanrate = scanrates[i];
		numchan = numchannels[i];
		for(unsigned int j=0; j < numchan; j++){

			//means stats first
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].mean);
					tempchannelname = statslog[k].channelname;//determine channel name
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);//compute mmm stats on current stat
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);//compute linear regression on current stat
			output << filename << ",";
			output << tempdata.size() << ",";
			output << scanrate << ",";
			output << j << ",";
			output << tempchannelname << ",";
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].mean == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].mean == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//maxes stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].max);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].max == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].max == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//mins stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].min);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].min == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].min == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//variances stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].variance);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].variance == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].variance == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//skews stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].skewness);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].skewness == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].skewness == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//kurtosi stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].kurtosis);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].kurtosis == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].kurtosis == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//cals stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].cal);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].cal == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].cal == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//psd peak frequencies stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].psdpeakfrequency);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].psdpeakfrequency == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].psdpeakfrequency == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//scale factor stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].scalefactor);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].scalefactor == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].scalefactor == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//offset stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].offset);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].offset == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].offset == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//damage stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].damage);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].damage == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].damage == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << ",";

			//flag stats
			tempdata.clear();
			unsigned int flagsum = 0;
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					flagsum += statslog[k].zerovarianceflag;
					flagsum += statslog[k].emptyscaleflag;
					flagsum += statslog[k].yellowlineflag;
					flagsum += statslog[k].redlineflag;
					flagsum += statslog[k].fullscaleflag;
					flagsum += statslog[k].hplpvarianceflag;
					flagsum += statslog[k].hplpmaxflag;
				}//end if
			}//end for
			output << flagsum << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].zerovarianceflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].emptyscaleflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].yellowlineflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].redlineflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].fullscaleflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].hplpvarianceflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";
			datafilename = "none";
			for(long int k=statslog.size()-1; k >= 0; k--){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].hplpmaxflag == 1){
					datafilename = statslog[k].filename;
					break;
				}//end if
			}//end for
			output << datafilename << ",";


			//psd peak frequencies stats
			tempdata.clear();
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j){
					tempdata.insert(tempdata.end(),statslog[k].psd99pctfrequency);
				}//end if
			}//end for
			mmmstats(tempdata,mean,max,min,variance,skew,kurtosis);
			LinearRegressionLeastSquares(tempdata,lryint,lrbeta,lrcorrelation,lrcharval);
			output << mean << ",";
			output << max << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].psd99pctfrequency == max){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << min << ",";
			datafilename = "none";
			for(unsigned int k=0; k < statslog.size(); k++){
				if(statslog[k].scanrate == scanrate && statslog[k].channel == j && statslog[k].psd99pctfrequency == min){
					datafilename = statslog[k].filename;
				}//end if
			}//end for
			output << datafilename << ",";
			output << variance << ",";
			output << skew << ",";
			output << kurtosis << ",";
			output << lryint << ",";
			output << lrbeta << ",";
			output << lrcorrelation << "\n";


		}//end for
	}//end for

	output.close();
}//end statsmmms



//The main program function.
int main(int argc, char* argv[])
{
	//calculate start time
	time_t starttime;
	time(&starttime);
	cout << "slaeversion:\t" << slaeversion << endl;
	cout << "starttime:\t" << starttime << endl;

	//load input data file
	cout << "filename:\t" << argv[1] << endl;
	cout << "reading stats log: ";
	readstatslog(argv[1]);
	cout << "done\n";

	cout << "stats lines:\t" << statslog.size() << endl;
	cout << "first mean:\t" << statslog[0].mean << endl;
	cout << "second mean:\t" << statslog[1].mean << endl;
	cout << "next last mean:\t" << statslog[statslog.size()-2].mean << endl;
	cout << "last mean:\t" << statslog[statslog.size()-1].mean << endl;

	//determine scan rates and channels
	determinescanratesandchannels();

	//compute stats on means and generate output
	statsmmms(argv[1]);

	//calculate analysis time
	time_t endtime;
	time(&endtime);
	cout << "endtime:\t" << endtime << endl;
	cout << "elapsed seconds:\t" << endtime-starttime << endl;
	cout << "elapsed minutes:\t" << (float)(endtime-starttime)/60.0 << endl;

	//calls external script/batch file
	//could zip up the output files or does anything else you want
	//it to do after analyzing a data file
	system("slae-postanalysis.bat");

	return 0;
}//end main


